function dataFileName = matrixFactorizationCF_sparse(R,dataFileName,opt)
% training the Matrix Factorization model 
% Inputs:
%  R: a sparse raing matrix, with 0 for the missing ratings
%  dataFileName: a mat File name where will store the model parameters:
%     mu, b_u,b_i,p_u, q_i,y_i w_ij, c_ij, S^k(i).
%  opt: optional, a structure containing the parameters of the algorighm,
%   opt may contain some of the fields as follows.
%   opt.similarityFile: mat file name containing the similarity matrix made
%         up of the similarity between each two items; set empty string, '', 
%         if there is no similarity file;
%   opt.classicalSVD: 1 or 0, whether to do classical SVD;
%                       r_ui= mu + b_u + b_i + <q_i, p_u>
%            default 0; the rating is estimated by the model proposed by
%            Y. Koren 2008;
%       In the case of classical SVD, opt.gamma_3, opt.lambda_8, opt.alpha
%       and opt.nNbr are not effective;    
%   opt.verbose     : 1 or 0, whether to put out the iteration process;
%   opt.lambda_6	: regularization parameter for bu, bi;
%   opt.lambda_7    : regularization parameter for q_i, p_u, y_j;
%   opt.lambda_8    : regularization parameter for w_ij, c_ij;
%   opt.gamma_1     : step size for bu, bi;
%   opt.gamma_2     : step size for q_i, p_u, y_j;
%   opt.gamma_3     : step size for w_ij, c_ij;
%   opt.sigma       : decreasing scalar for step sizes at each iteration
%   opt.alpha       : the role of nearest neighbors
%   opt.nNbr        : neighborhood size
%   opt.ite         : iteration number 
%   opt.nf          : factor space dimension
%   opt.threshold   : threshold to calculate similarity between items
% Outputs:
%  * Notations: 
%      u    : user u; 
%      i    : item i;
%      m    : number of users;
%      n    : number of items;
%     R(u)  : item indices for which ratings by user u are available;
%     N(u)  : all item indices for which user u provided an implicit preference;    
%    S^k(i) : the set of k items most similar to item i;
%   R^k(i;u): R(u) �� S^k(i);
%   N^k(i;u): N(u) �� S^k(i);
%    b_ui   : b_ui = mu + b_u + b_i baseline estimate, accounting for the user
%               and item effects;
%   * Model parameters:
%      mu      :  overall average rating, mu is a scalar;
%      b_u,b_i : observed deviations of user u and item i; note that b_u,
%                    b_i are entries of two different vectors
%    p_u in R^f: user-factors vector; u = 1,...,m
%    q_i in R^f: item-factors vector (explicit); i = 1,...,n
%    y_i in R^f: item-factors vector (implicit); i = 1,...,n
%      w_ij    : weight representing offsets to baseline estimation; 
%                  i = 1,...,n; j in R^k(i;u)
%      c_ij    : offsets added to baseline estimates which is induced by
%                  implicit feedback i = 1,...,n; j in N^k(i;u)
% Reference: Y. Koren. Factorization meets the Neighborhood: a multifaced
% Collaborative Filtering model. KDD. 2008 

debug = 0; % 1 or 0, whether debug mode
BETA = 0;
I = (R~=BETA);
trainSize = nnz(I);
[nUser nItem] = size(R);

% Step 0. Initialization
%  0.1 set parameters 
  % regularization parameters
if nargin<=2
    opt=struct();
end
opt = completeArg(opt,{'lambda_6','lambda_7','lambda_8','gamma_1','gamma_2','gamma_3',...
    'sigma','alpha','nNbr','nf','ite_num','threshold','classicalSVD','similarityFile','verbose'},...
    {.005, .02, .02, .025, .025, .01, .9, .9, 200, 10, 10, 200, 0, '',1});
  % 0.2 initialize b_u, b_i, p_u, q_i, w_ij, c_ij
bu = zeros(nUser,1);
bi = zeros(nItem,1);
P = 0.2*ones(opt.nf,nUser); % P(:,u) stores the vector p_u
Q = 0.2*ones(opt.nf,nItem); % Q(:,i) stores the vector q_i
Y = [];
W = [];
C = [];
if ~opt.classicalSVD
    Y = zeros(opt.nf,nItem); % Y(:,i) stores the vector y_i
    W =  zeros(nItem,opt.nNbr); % W(i,j) stores the parameter w_ij
    C =  zeros(nItem,opt.nNbr); % C(i,j) stores the parameter c_ij
end
  
  % 0.3 calculate mu, and S^k(i)
mu = sum(sum(R.*I))/trainSize;  
if ~opt.classicalSVD  % classical SVD do not need S^k(i)
    if isfield(opt,'similarityFile')&&~isempty(opt.similarityFile)
        load(opt.similarityFile,'S');
    else
        S = similarity_item(R,opt.nNbr,opt.threshold); % calculate the similarity between each two items
        save(sprintf('../iteration/similarity_item_%s.mat',datestr(date)),'S');
    end
    [Sk_val Sk] = sort(S,2,'descend');
    Sk = Sk(1:opt.nNbr,:)'; % just store opt.nNbr most similar items
else 
    Sk = [];
end
  
% Step 1. Iteration
[u_id_v i_id_v] = find(I);
    % u_id_v(i), i_id_v(i) are the user index and item index for the i-th
    % known rating, "_v" means this is a indentifier of a vector
lambda_6 = opt.lambda_6; 
lambda_7 = opt.lambda_7;
lambda_8 = opt.lambda_8;
alpha = opt.alpha;

ite_num = opt.ite_num;
nf = opt.nf;
if debug % debug mode
    ite_num = 1;
    nf = 1;
end

for ite=1:ite_num
    gamma_1 = opt.gamma_1; 
    gamma_2 = opt.gamma_2;
    gamma_3 = opt.gamma_3;
    for dim = 1:nf
        if opt.verbose
            fprintf('Matrix Factorization CF, iteration %d\t dimension %d\n',ite,dim);
        end
        % 1.1 shuffle the order of the entries in K, K:={(u,j) | R(u,i) is known}
        id_v = randperm(trainSize);
        if debug
            id_v = 1:trainSize; 
                % in debug mode, do not perform the rand permutation,
                %   in order to locate the user and item
        end
        e_ui_ave = 0;        
        % 1.2 train the model over all the entries in K    
        for jj=1:trainSize  
%         for jj=1:200
            if debug
                fprintf('%d:  e_ui_ave:\t %.4f\n',jj,e_ui_ave/trainSize);
            end
            if isnan(e_ui_ave)
                break
            end
            % training the model on the entry (u,i) 
            id = id_v(jj);
            u = u_id_v(id); 
            i = i_id_v(id);
            % 1.2.1 obtain R(u), N(u) and R^k(i;u), N^k(i;u), In current application
              % R(u)=N(u), R^k(i;u) = N^k(i;u);
            len_Nu = 0;
            if ~opt.classicalSVD
                Ru = find(I(u,:)); % Ru: R(u)
                Nu = Ru;           % Nu: N(u)
                len_Nu = length(Nu);
                [Rk_iu Rk_id]= intersect(Sk(i,:),Ru); % Rk_iu: R^k(i;u)
                Nk_iu = Rk_iu;                        % Nk_iu: N^k(i;u)
                Nk_id = Rk_id;                          
                len_Nk_iu = length(Nk_iu);
                len_Rk_iu = length(Rk_iu);
            end
            % 1.2.2 calculate the estimation for r_ui
            rr_ui =  cal_rr_ui();
            % 1.2.3 calculate e_ui
            e_ui = R(u,i) - rr_ui;
            e_ui_ave = e_ui_ave + abs(e_ui);
            % 1.2.4 update bu, bi, q_i(dim), p_u(dim), y_j(dim), w_ij, and c_ij
                % (1) bu_u  = bu_u + gamma_1*(e_ui-lambda_6*bu_u)
            bu(u) = bu(u) + gamma_1*(e_ui-lambda_6*bu(u));        
                % (2) bi_i  = bi_i + gamma_1*(e_ui-lambda_6*bi_i)
            bi(i) = bi(i) + gamma_1*(e_ui-lambda_6*bi(i));
                % (3) q_i = q_i + gamma_2*(e_ui(p_u+|N(u)|^(-1/2)*sum_{j in N(u)}y_j )-lambda_7*q_i)
            temp = Q(dim,i);
            if ~opt.classicalSVD && len_Nu>0
                Q(dim,i) = Q(dim,i) + gamma_2*(e_ui*(P(dim,u)+weight(len_Nu)*sum(Y(dim,Nu),2))...
                    -lambda_7*Q(dim,i));                        
            else % do classical SVD
                % q_i = q_i + gamma_2*(e_ui * p_u -lambda_7*q_i)
                Q(dim,i) = Q(dim,i) + gamma_2*(e_ui*P(dim,u)-lambda_7*Q(dim,i));
            end
                % (4) p_u = p_u + gamma_2*(e_ui*q_i-lambda_7*p_u)
            P(dim,u) = P(dim,u) + gamma_2*(e_ui*temp-lambda_7*P(dim,u));
            if ~opt.classicalSVD
                % (5) for all j\in N(u), y_j = y_j + gamma_2*(e_ui*|N(u)|^(-1/2)*q_i-lambda_7*y_j)
                if len_Nu>0  
                    Y(dim,Nu) = (1-gamma_2*lambda_7)*Y(dim,Nu) + ...
                        repmat(gamma_2*e_ui*weight(len_Nu)*Q(dim,i),1,len_Nu);
                end
                % (6) for all j\in R^k(i;u), w_ij = w_ij + gamma_3*(alpha*|R^k(i;u)|^(-1/2))*e_ui*(r_uj-b_uj)-lambda_8*w_ij)
                if len_Rk_iu>0
                    W(i,Rk_id) = (1-gamma_3*lambda_8)*W(i,Rk_id) + ...
                        gamma_3*alpha*weight(len_Rk_iu)*e_ui*(-mu-bu(u)-bi(Rk_iu)'+R(u,Rk_iu));   
                end
                % for all j\in N^k(i;u), c_ij = c_ij + gamma_3*(alpha*|R^k(i;u)|^(-1/2))*e_ui-lambda_8*c_ij)
                if len_Nk_iu>0
                    C(i,Nk_id) = (1-gamma_3*lambda_8)*C(i,Nk_id) + ...
                        gamma_3*alpha*weight(len_Nk_iu)*e_ui;
                end
            end
        end % end of the loop for ii=1:trainsize    
        % 1.3 reduce the step sizes
        gamma_1 = gamma_1 * opt.sigma;
        gamma_2 = gamma_2 * opt.sigma;
        gamma_3 = gamma_3 * opt.sigma;
        if opt.verbose
            fprintf('e_ui_ave:\t %.4f\n',e_ui_ave/trainSize);
        end
    end
      
end
% Step 2. save the trained parameters to file
save(dataFileName,'R','BETA','mu','bu','bi','P','Q','Y','W','C','Sk','alpha','opt');    


    % embeded function
    function r_val =  cal_rr_ui()
        % calculate the estimation for the rating r_ui
        %  rr_ui= mu + b_u + b_i + q_i'*(p_u + |N(u)|^(-0.5) * sum_{j\in N(u)}y_j)
        %       + alpha*|R^k(i;u)|^(-0.5) * sum_{j\in R^k(i;u)} (r_uj - b_uj)*w_ij
        %       + alpha*|N^k(i;u)|^(-0.5) * sum_{j\in N^k(i;u)} c_ij        
        t2 = 0; 
        t3 = 0;        
        if ~opt.classicalSVD && len_Nu>0
            t1 = Q(1:dim,i)'* (P(1:dim,u) + weight(len_Nu)*sum(Y(1:dim,Nu),2) );
        else
            t1 = Q(1:dim,i)'* P(1:dim,u);
        end
        if ~opt.classicalSVD
            if len_Rk_iu
                t2 = alpha*weight(len_Rk_iu)*W(i,Rk_id)*(R(u,Rk_iu)'-(mu+bu(u)+bi(Rk_iu)));
                    % note that W(i,Rk_id) is a row vector
            end
            if len_Nk_iu
                t3 = alpha*weight(len_Nk_iu)*sum(C(i,Nk_id));
            end
        end
        r_val = mu + bu(u) + bi(i) + t1 + t2 + t3;
        if debug
        	fprintf(1,'t1: %f, t2: %e, t3: %e, len_Rk_iu: %d\n',...
                t1,t2,t3,length(Rk_iu));
        end
    end

end % End of the function MatrixFactorizationCF_sparse()
