function r = nearest_neighbor_item_base(R,BETA,users,items,max_num_neighbor,flag_all_entries,nbrSize_similarity,threshold)
% item based Nearest Neighbor method

% Inputs & Outputs:
%  R, BETA:  raing matrix, with BETA (a real number) the missing ratings;
%  users, items, r: if flag_all_entries=0, r is a column vector, r(i) is 
%    the predicted rating for the user "users(i)" on the item "item(i)"; if
%    some entry cann't be covered (predicted), the prediction will be set
%    as DEFAULT_RATING;
%  max_num_neighbor: optional, a positive integer indicating the upper bound of
%    the number of neighbors, the default value is 50;
%  flag_all_entries: optional, indicate whether to indicate all the missing ratings in 
%    matrix R, if the value is not 0, the function will predict all the
%    missing raings and the output r is a matrix with the size the same as matrix R. 
%  nbrSize_similarity: optional, a positive integer, the number of similarities
%    stored for each item when calculating the similarites with other items
%  threshold: optional, a positive integer indicating the significance
%    threshold, default value is 50;
nTest = length(users);
[nUser nItem] = size(R);
I = (R~=BETA);

minRating = min(R(I)); % minimum rating
maxRating = max(R(I)); % maximum rating
DEFAULT_RATING = minRating+(maxRating-minRating)/2; %the predicted ratings can't be covered (no ratings on 
        % the user or on the item in the rating matrix) will be set as this value

if nargin<=4
    max_num_neighbor = 50;
end
if nargin<=5
    flag_all_entries = 0;
end
if nargin<=6
    nbrSize_similarity = 10*max_num_neighbor;         
end
if nargin<=7
    threshold = 50;
end
if ~flag_all_entries 
    if nTest~=length(items)
        error('the arguments users and items should be equal length.');
    end
end
flag_use_negative_similarity = 1; % whether to use negative similarities

if flag_all_entries
    r = R;
    r = full(r);
else
    r = zeros(nTest,1);
end

% calculate the similarity (Pearson correlation) between users

S = similarity_item(R,nbrSize_similarity,threshold);
save(sprintf('../iteration/similarity_item_train_%s.mat',datestr(date)),...
    'S','nbrSize_similarity','threshold');
%load('../iteration/similarity_item_train.mat','S','nbrSize_similarity');

if flag_use_negative_similarity 
    max_similarity = ceil(max(max(abs(S)))); % maximum similarity magnitude
    offset = max_similarity + 1;
else
    offset = 0;
end
 
if max_num_neighbor>nbrSize_similarity
    max_num_neighbor = nbrSize_similarity;
end

% set the items for which the ratings to be predicted
if ~flag_all_entries
    [t id_t]= sort(items);
    [item_v id_item_v] = unique(t,'first');    
else
    item_v = 1:nItem;
end    
for ii=1:length(item_v)
    % predict for the rating for item_v(ii) of given items
    id_item = item_v(ii);
    % set users to be predicted
    if ~flag_all_entries
        if ii<length(item_v)
            sec_item = id_item_v(ii+1)-1;
        else
            sec_item = length(item_v);
        end
        i_users = items(id_t(id_item_v(ii):sec_item)); % users to be predicted
        i_users_ind = id_t(id_item_v(ii):sec_item);    % indices of users to be predicted
    else
        i_users = find(~I(:,id_item));
    end    
    if ~flag_all_entries && id_item>nItem         % item index exceedes the item number 
        r(id_t(id_item_v(ii):sec_item)) = DEFAULT_RATING;
        continue;
    elseif ~flag_all_entries &&~nnz(I(:,id_item)) % no user has rated item id_item 
        r(id_t(id_item_v(ii):sec_item)) = DEFAULT_RATING;
        continue;
    elseif flag_all_entries &&~nnz(I(:,id_item)) % no user has rated item id_item 
        r(i_users,id_item) = DEFAULT_RATING;
        continue;
    end
    if ~flag_all_entries && max(i_users)>nUser % some indices of users exceeded the user number 
        i_out = find(users(i_users_ind)>nUser);
        r(id_item_v(ii)+i_out-1)= DEFAULT_RATING; % set these entris the default rating
        i_users = setdiff(i_users,i_users(i_users_ind(i_out)));
        i_users_ind = setdiff(i_users_ind,i_users_ind(i_out));
        if isempty(i_users)
            continue;
        end
    end
    ni_users = length(i_users);
    
    % find the top-N similar items with item item_v(ii) for each users
    %   to be predicted   
    V = repmat( S(id_item,:),ni_users,1).*I(i_users,:);  
    II = V~=0;    
    
    if  nItem > max_num_neighbor
         % just hold the top-N similarities
        V(II) = V(II)+offset; % add the nonzero elements an offset; 
        [U]= sort(V,2,'descend'); % sort the similarities by descent order 
        V(II) =  V(II)-offset;  % reset the nonzero elements, neglect the zero elements in V when perform ordering;
                                            % notice that U has not been reset;           
        B = spdiags(U(:,max_num_neighbor)-offset,0,ni_users,ni_users)*II;
        V(V<B) = 0;          % V( V<=repmat(U(max_num_neighbor,:),nItem,1) ) = 0;         
    end       
    
      % calculate the weighted average ratings on top-N similar users 
    sum_V_temp = sum(abs(V),2);
    sum_V_temp = sum_V_temp + ~sum_V_temp; % deal with the case that sum_V_temp(j) = 0, for some j;
    act = sum(R(i_users,:).*V,2)./ sum_V_temp;
        
    if ~flag_all_entries
        r(i_users_ind) =  act;
    else
        r(i_users,id_item) =  act;         
    end
    %%%%
    if mod(ii,10)==1
        fprintf('predicted the ratings for item %d\n',id_item);
    end
    %%%%
end

r = r + ~r*DEFAULT_RATING; % set the ratings 0 to default rating
r(r<minRating) = minRating;
r(r>maxRating) = maxRating;