function r = nearest_neighbor_prediction(R,BETA,users,items,max_num_neighbor,flag_all_entries)
% user based nearest neighbor method
% Inputs & Outputs:
%  R, BETA:  raing matrix, with BETA (a real number) the missing ratings;
%  users, items, r: if flag_all_entries=0, r is a column vector, r(i) is 
%    the predicted rating for the user "users(i)" on the item "item(i)"; if
%    some entry cann't be covered (predicted), the prediction will be set
%    as DEFAULT_RATING;
%  max_num_neighbor: optional, a positive integer indicating the upper bound of
%    the number of neighbors, the default value is 50;
%  flag_all_entries: indicate whether to indicate all the missing ratings in 
%    matrix R, if the value is not 0, the function will predict all the
%    missing raings and the output r is a matrix with the size the same as matrix R. 

% Reference:
%  [1] Xiaoyuan Su, Taghi M. Khoshgoftaar, A Survey of Collaborative Filtering
%  Techniques. Advances in Artificial Intelligence. 2009.
%  [2] J. Herlocker, J. A. Konstan, J. Riedl, An empirical analysis of
%  design choices in neighborhood-based collaborative filtering
%  algorithms. Information Retrieval, 5, 287--310, 2002.

nTest = length(users);
[nUser nItem] = size(R);
I = (R~=BETA);

minRating = min(R(I)); % minimum rating
maxRating = max(R(I)); % maximum rating
DEFAULT_RATING = minRating+(maxRating-minRating)/2; %the predicted ratings can't be covered (no ratings on 
        % the user or on the item in the rating matrix) will be set as this value

if nargin<=4
    max_num_neighbor = 50;
end
if nargin<=5
    flag_all_entries = 0;
end
if ~flag_all_entries 
    if nTest~=length(items)
        error('the arguments users and items should be equal length.');
    end
end
flag_use_negative_similarity = 1; % whether to use negative similarities

if flag_all_entries
    r = R;
    r = full(r);
else
    r = zeros(nTest,1);
end

% users' average ratings
aveR_vec = userAverage(R,BETA);

% calculate the similarity (Pearson correlation) between users
nbrSize_similarity = max(max_num_neighbor,round(size(R,1)*0.3));
    % number of maximum neighbors when calculate the similarity between
    % users (at most store nbrSize_similarity values for each user)
    
%S = similarity_user(R,max_num_neighbor);
S = similarity_user(R,nbrSize_similarity);
save(sprintf('../iteration/similarity_user_train_%s.mat',datestr(date)),...
    'S','nbrSize_similarity');
% load('../iteration/similarity_user_train.mat','S','nbrSize_similarity');
% load('../iteration/similarity.mat','W');
% S = W;
% S = S - eye(nUser);
% nbrSize_similarity = 943;

if flag_use_negative_similarity 
    max_similarity = ceil(max(max(abs(S)))); % maximum similarity magnitude
    offset = max_similarity + 1;
else
    offset = 0;
end
 
% % % if max_num_neighbor>nbrSize_similarity
% % %     max_num_neighbor = nbrSize_similarity;
% % % end

% set the users for whom  the ratings to be predicted
if ~flag_all_entries
    [u iu]= sort(users);
    [user iuser] = unique(u,'first');    
else
    user = 1:nUser;
end    

% if matlabpool('size') == 0
%         matlabpool open % open pool of matlab sessions for parallelcomputation
% end
% fprintf('matlabpool size: %d\n',matlabpool('size'));
    
for ii=1:length(user)
    % predict for the rating for user "user(ii)" on given items
    id_user = user(ii);
    % set items to be predicted
    if ~flag_all_entries
        if ii<length(user)
            sec_iuser = iuser(ii+1)-1;
        else
            sec_iuser = length(users);
        end
        i_items = items(iu(iuser(ii):sec_iuser)); % items to be predicted
        i_items_ind = iu(iuser(ii):sec_iuser); % indices (in "items") of items to be predicted
    else
        i_items = find(~I(id_user,:)); % items to be predicted
    end    
    %ni_items = length(i_items);
    if ~flag_all_entries && id_user>nUser % user index exceedes the user number 
        r(iu(iuser(ii):sec_iuser)) = DEFAULT_RATING;
        continue;
    elseif ~flag_all_entries &&~nnz(I(id_user,:)) % user id_user has rated no item
        r(iu(iuser(ii):sec_iuser)) = DEFAULT_RATING;
        continue;
    elseif flag_all_entries &&~nnz(I(id_user,:)) % user id_user has rated no item
        r(id_user,i_items) = DEFAULT_RATING;
        continue;
    end
    if ~flag_all_entries && max(i_items)>nItem % some indices of items exceeded the item number 
        i_out = find(items(i_items_ind)>nItem);
        r(iuser(ii)+i_out-1)= DEFAULT_RATING; 
        i_items = setdiff(i_items,i_items(i_items_ind(i_out)));
        i_items_ind = setdiff(i_items_ind,i_items_ind(i_out));
        if isempty(i_items)
            continue;
        end
    end
    ni_items = length(i_items);
    % find the top-N similar users for user iuser(ii) on each items to be predicted   
    V = repmat( S(id_user,:)',1,ni_items ).*I(:,i_items);  
    II = V~=0;    
    
    if  nUser > max_num_neighbor
         % just hold the top-N similarities
        V(II) = V(II)+offset; % add the nonzero elements an offset; 
        [U]= sort(V,1,'descend'); % sort the similarities by descent order 
        V(II) =  V(II)-offset;  % reset the nonzero elements, neglect the zero elements in V when perform ordering;
                                            % notice that U has not been reset;           
        B = II*spdiags(U(max_num_neighbor,:)'-offset,0,ni_items,ni_items);
        V(V<B) = 0;          % V( V<=repmat(U(max_num_neighbor,:),nUser,1) ) = 0;         
    end       
    
      % calculate the weighted average ratings on top-N similar users 
    sum_V_temp = sum(abs(V));
    sum_V_temp = sum_V_temp + ~sum_V_temp; % deal with the case that sum_V_temp(j) = 0, for some j;
    % act = sum((R(:,i_items)-aveR(:,1:ni_items)).*V)./sum_V_temp; %%% storage waste, may be refined;
    
    B = spdiags(aveR_vec,0,nUser,nUser)*II;
    Z = R(:,i_items);
    Z(II) = Z(II) - B(II);
    act = sum(Z.*V)./sum_V_temp; 
    
    if ~flag_all_entries
        r(i_items_ind) =  aveR_vec(id_user) + act';
    else
        r(id_user,i_items) =  aveR_vec(id_user) + act;         
    end
    %%%%
    if mod(ii,100)==1
        fprintf('predicted the ratings for user %d\n',id_user);
    end
    %%%%
end
%matlabpool close
    
r = r + ~r*DEFAULT_RATING; % set the ratings 0 to default rating
r(r<minRating) = minRating;
r(r>maxRating) = maxRating;
    
 

