function W = similarity_item(R,nbr_size,threshold)
% Calculate Pearson correlation between items
%
% Inputs:
%  R:  raing matrix, a sparse matrix
%  nbr_size: optional, neighbor size, for each item, store the most similar
%    nbr_size items, default value is 100;
%  threshold: optional, a positive integer indicating the significance
%    threshold, default value is 50;
% Outputs:
%  W: a sparse matrix, W(i,:) store nbr_size most similar nbr_size
%    items; others entries will be set 0, if W(i,j)~=0, it indicate the
%    Pearson correlation between item i and item j; if items i and j have
%    no common ratings, the correlation W(i,j) will be set 0;
%
% Reference:
%  [1] J. Ben Schafer, Dan Frankowski, Jon Herlocker et al., Collaborative
%  Filtering Recommendation Systems. P. Brusilovsky, A. Kobsa, and W.
%  Nejdl(Eds.) The Adaptive Web. pp. 291--324, 2007. Springer-Verlag Berlin
%  Heidelberg 2007.
%  [2] J. Herlocker, J. A. Konstan, J. Riedl, An empirical analysis of
%  design choices in neighborhood-based collaborative filtering
%  algorithms. Information Retrieval, 5, 287--310, 2002.

if nargin<=1
    nbr_size = 100;
end
if nargin<=2
    threshold = 50;
end

flag_significance_weight = 1; % use significance weighting
BETA = 0;
[nUser nItem] = size(R);

if nbr_size<nItem
    W = sparse(nItem,nItem);
else
    W = zeros(nItem,nItem);
end

I = (R~=BETA);
[u] = userAverage(R,BETA);

for id_u = 1:nUser
    R(id_u,I(id_u,:)~=0) = R(id_u,I(id_u,:)~=0) - u(id_u);
end
offset = 2; % note that the range of the similarities is [-1, 1], set offset>1;
fprintf('\nStart to calulate similarity between items.\n'); %%%%%
for ii = 1:nItem
    % given item ii, calculate the Person correlation between item ii and
    %   item jj, for all jj>ii     
    if nnz(I(:,ii)) % ratings for item ii is not empty
        %t = ii+1:nItem;
        I2 = I.*repmat(I(:,ii),1,nItem);
        R2 = repmat(R(:,ii),1,nItem);
        norm_temp = sqrt(sum(R2.*R2).*sum(R.*R.*I2));
        w_v = sum(R2.*R.*I2)./(norm_temp+~norm_temp); 
                    % "+~norm_temp": deal with the case that norm_temp(jj)==0
        if flag_significance_weight
            num_vec = sum(I2);
            w_v = min(num_vec/threshold,1).*w_v;
        end
        if nbr_size<nItem
            w_v(ii) = 0;  % exclude the item itself from the most similar nbr_size items            
            w_v(w_v~=0) = w_v(w_v~=0) + offset;
            [w_v iw_v] = sort(w_v,'descend');
            w_v(w_v~=0) = w_v(w_v~=0) - offset; % neglect the zero similarities, consider the negative similarities
            W(ii,iw_v(1:nbr_size)) = w_v(1:nbr_size);
        else
            w_v(ii) = 0;
            W(ii,:) = w_v;
        end
    end
    if ~mod(ii,200) %%%%%%
        fprintf('%d\n',ii);
    end
end
fprintf('\nFinished to calulate similarity between items.\n'); %%%%%
