function W = similarity_user(R,nbr_size,threshold)
% Calculate Pearson correlation between users
%
% Inputs:
%  R:  raing matrix, a sparse matrix
%  nbr_size: optional, neighbor size, for each user, store the most similar
%    nbr_size users, default value is 100;
%  threshold: optional, a positive integer indicating the significance
%    threshold, default value is 50;
% Outputs:
%  W: a sparse matrix, W(i,:) store nbr_size most similar nbr_size
%    items; others entries will be set 0, if W(i,j)~=0, it indicate the
%    Pearson correlation between user i and user j; if user i and j have
%    no common ratings, the correlation W(i,j) will be set 0;

% Reference:
%  [1] Xiaoyuan Su, Taghi M. Khoshgoftaar, A Survey of Collaborative Filtering
%  Techniques. Advances in Artificial Intelligence. 2009.
%  [2] J. Herlocker, J. A. Konstan, J. Riedl, An empirical analysis of
%  design choices in neighborhood-based collaborative filtering
%  algorithms. Information Retrieval, 5, 287--310, 2002.

if nargin<=1
    nbr_size = 100;
end
if nargin<=2
    threshold = 50;
end

flag_significance_weight = 1; % use significance weighting
BETA = 0;
nUser = size(R,1);
if nbr_size<nUser
    W = sparse(nUser,nUser);
else
    W = zeros(nUser,nUser);
end
I = (R~=BETA);
[u] = userAverage(R,BETA);

% tic
% for id_u = 1:nUser
%     R(id_u,I(id_u,:)~=0) = R(id_u,I(id_u,:)~=0) - u(id_u);
% end
% t1 = toc

% tic
    B = spdiags(u,0,nUser,nUser)*I;
    R(I) = R(I)-B(I);
% t2 = toc
offset = 2; % note that the range of the similarities is [-1, 1];
fprintf('\nStart to calulate similarity between users.\n'); %%%%%
for ii = 1:nUser
    % given user ii, calculate the Person correlation between user ii and
    %   user jj, for all jj>ii     
    if nnz(I(ii,:)) % ratings for user ii is not empty
        %t = ii+1:nItem;
        I2 = I.*repmat(I(ii,:),nUser,1);
        R2 = repmat(R(ii,:),nUser,1);
        norm_temp = sqrt(sum(R2.*R2,2).*sum(R.*R.*I2,2));
        w_v = sum(R2.*R.*I2,2)./(norm_temp+~norm_temp); 
                    % "+~norm_temp": deal with the case that norm_temp(jj)==0
        if flag_significance_weight
            num_vec = sum(I2,2);
            w_v = min(num_vec/threshold,1).*w_v;
        end        
        if nbr_size<nUser
            w_v(ii) = 0;  % exclude the item itself from the most similar nbr_size items            
            w_v(w_v~=0) = w_v(w_v~=0) + offset;
            [w_v iw_v] = sort(w_v,'descend');
            w_v(w_v~=0) = w_v(w_v~=0) - offset; % neglect the zero similarities, consider the negative similarities
            W(ii,iw_v(1:nbr_size)) = w_v(1:nbr_size)';
        else
            w_v(ii) = 0;
            W(ii,:) = w_v';
        end
    end
    if ~mod(ii,200) %%%%%%
        fprintf('%d\n',ii);
    end
end
fprintf('\nFinished to calulate similarity between users.\n'); %%%%%
