function [MAE_opt RMSE_opt] = Chaos(chaosFile,R_quiz,R_test,Quiz,Test,...
    act_quiz,act_test,rel_quiz,rel_test,index_MAE,arg)
% regress the prediction results to genereate a set of new predictions on
% both quiz set and test set
% 
% Inputs: 
%   chaosFile: a mat file name, save the calculation results to this file;
%   R_quiz, R_test: two matrices with same column number with each column
%       containing a group of prediction results by one model on Quiz set, and
%       Test set, respectively; the first column store the predictions with
%       the lowest whole MAE over all the methods;
%   Quiz, Test: two sparse matrices containing the ratings to be tested;
%   act_quiz, act_test: column vectors, activities on the entries of Quiz
%      an Test set; 
%   rel_quiz, rel_test: column vectors, relatedness on the entries of Quiz
%      an Test set; 
%   index_MAE: a matrix with two columns to estimate the relationship between
%       relatedness and MAE, such as
%       [ .010 .60;
%         .012 .65;
%         .016 .70;
%         .021 .80
%         .025 .90];    
%       the first column is relatedness and the second column indicates 
%       corresponding MAE; for example, the index_MAE above means that 
%       relatedness between (-inf .010] corresponds MAE = .60 and (.010,
%       .12] corresponds MAE = .65, etc.
%  arg: a structure containing arguments
%    arg.weight_v: Optional, a  vector with 3 elements, indication the
%       percent of white, gray and black sheep; 
%    arg.blue_activiry: Optional, a positive integer, entries with
%      activities less than this given value will be separated as blue sheep
%    arg.l_rating: Optional, lower bound of the predicted rating, default 1; 
%    arg.r_rating: Optional, upper bound of the predicted rating, default 5; 
%    arg.padMAE: Optional, a scalar located in in [0,0.5), the 2nd column
%       of index_MAE will be added to this quantities before operating
%       clips, default 0; 
%    arg.indexMode: Optional, 'ascend' or 'descend': indicating the value of
%      confidence measure (e.g. relatedness) is ascending or descending
%      from White Sheep to Black Sheep; 
% Outputs:
%  MAE_opt, RMSE_opt: MAE and RMSE of chaos_clip method (first do
%    regression, then operate rounding) on Test set

%  1. Initialization
[nQuiz nModel]= size(R_quiz);
nTest = size(R_test,1);
if size(R_test,2)~=nModel
    error('R_quiz and R_test should have the same column.');
end
% separate the Test set into 2 subset: Test_1, Test_2
%   Test_1 is used to observe the accuracy of the linear regression model;
%   Test_2 is used to evaluate the performace of the linear regression model;
sampleRate_Test_2 = 1/2; % nnz(Test_2)/nnz(Test)
[Test_1 Test_2] = quiz(Test,sampleRate_Test_2);

% assign the ratings on Quiz, Test_1 and Test_2

ind_quiz_v = find(Quiz);
ind_test_v = find(Test);
ind_test1_v = find(Test_1);
ind_test2_v = find(Test_2);

rr_quiz = Quiz(ind_quiz_v);
rr_test1 = Test(ind_test1_v);
rr_test2 = Test(ind_test2_v);

% assign the activities and relatedness on Quiz, Test_1 and Test_2
[temp ind_test1_test] = ismember(ind_test1_v,ind_test_v);
[temp ind_test2_test] = ismember(ind_test2_v,ind_test_v);
act_test1 = act_test(ind_test1_test);
act_test2 = act_test(ind_test2_test);
rel_test1 = rel_test(ind_test1_test);
rel_test2 = rel_test(ind_test2_test);

save(chaosFile,'R_quiz','R_test','Quiz','Test','act_quiz','act_test','rel_quiz','rel_test','index_MAE');
%save(chaosFile,'user_quiz_v','item_quiz_v','rr_quiz','user_test_v','item_test_v','rr_test','-append');
save(chaosFile,'act_test1','act_test2','rel_test1','rel_test2',...
    'ind_quiz_v','rr_quiz','ind_test1_v','ind_test2_v','rr_test1','rr_test2','-append');

% 2. Seperate the entries (sheep) on Quiz and Test set into 4 groups, respectively;
   
whole_q = (1:nQuiz)';
[white_q gray_q black_q blue_q] = white_gray_black_blue_sheep(whole_q,act_quiz,rel_quiz,arg);
whole_t = (1:nTest)';
[white_t gray_t black_t blue_t] = white_gray_black_blue_sheep(whole_t,act_test,rel_test,arg);
whole_t1 = (1:length(act_test1))';
[white_t1 gray_t1 black_t1 blue_t1] = white_gray_black_blue_sheep(whole_t1,...
    act_test1,rel_test1,arg);
whole_t2 = (1:length(act_test2))';
[white_t2 gray_t2 black_t2 blue_t2] = white_gray_black_blue_sheep(whole_t2,...
    act_test2,rel_test2,arg);
save(chaosFile,'white_q','gray_q','black_q','blue_q',...
    'white_t1','gray_t1','black_t1','blue_t1',...
    'white_t2','gray_t2','black_t2','blue_t2',...
    'arg','-append');
fprintf('* Finished to seperate the sheeps to 4 groups.\n');

% 3. linear fitting to find a coefficients on white, gray, black and blue
%   sheep based on Quiz set
group_v_quiz = {'white_q','gray_q','black_q','blue_q','whole_q'};
len_group_v_quiz = length(group_v_quiz);
Alpha = zeros(nModel,len_group_v_quiz); % each column of Alpha store a group of coefficients
% lambda = [0.03 0.03 0.10 0.12 0.04];
% lb = -0.05*ones(nModel,1); ub = 1.0*ones(nModel,1);         % bound: lb <= x <= beq;
% A_neq = 1.0*ones(1,nModel); b_neq = 1.20;     % A_neq * x <= b_neq
% Aeq = []; beq = [];
% x0 = ones(nModel,1)/nModel; % initial point
% opt = optimset('LargeScale','off');
%w = ones(nModel,1);
lambdaStop = 1.0E-4; % given a number near 0
%kappa = 1;
for ii=1:len_group_v_quiz
    id_v = eval(group_v_quiz{ii});       
    if ~isempty(id_v)
        %x = ridge_regression_con(R_quiz(id_v,:),rr_quiz(id_v),A_neq,b_neq,Aeq,beq,lb,ub,x0,opt,lambda(ii));
%         x = SolveGeneralLasso(R_quiz(id_v,:),rr_quiz(id_v),lambdaStop);
        x = SolveGeneralLasso(R_quiz(id_v,:),rr_quiz(id_v),zeros(size(R_quiz,2),1),lambdaStop,1);
        Alpha(:,ii) = x;
    end    
end
% save(chaosFile,'Alpha','group_v_quiz','lambda','lb','ub','A_neq','b_neq','Aeq','beq','x0','opt','-append');
save(chaosFile,'Alpha','group_v_quiz','lambdaStop','-append');
fprintf('* Finished to do linear fitting.\n');

% 4. Predict the ratings of the each group of sheep in Quiz and Test Set, respectively;
r_chaos_quiz_1 = zeros(nQuiz,1);
r_chaos_test_1 = zeros(nTest,1); % predictions generated by regressions on various groups 
group_v_test = {'white_t','gray_t','black_t','blue_t','whole_t'};
group_v_test1 = {'white_t1','gray_t1','black_t1','blue_t1','whole_t1'};
group_v_test2 = {'white_t2','gray_t2','black_t2','blue_t2','whole_t2'};

len_group_v_test = length(group_v_test);
  % on quiz set
for ii=1:len_group_v_quiz-1
    id_v = eval(group_v_quiz{ii});
    if ~isempty(id_v)
        r_chaos_quiz_1(id_v) = R_quiz(id_v,:)*Alpha(:,ii);
    end
end
    % predictions generated by linear combination on the whole
r_chaos_quiz_0 = R_quiz*Alpha(:,len_group_v_quiz);
    
  % on test set
for ii=1:len_group_v_test-1
    id_v = eval(group_v_test{ii});
    if ~isempty(id_v)
        r_chaos_test_1(id_v) = R_test(id_v,:)*Alpha(:,ii);
    end
end
    % predictions generated by linear combination on the whole
r_chaos_test_0 = R_test*Alpha(:,len_group_v_test);
    
% clip the predictions by Chaos
getArg(arg,'l_rating',1); 
getArg(arg,'r_rating',5);
getArg(arg,'padMAE',0);



    % append the prediction results to R_quiz and R_test    
R_quiz = [R_quiz r_chaos_quiz_1 r_chaos_quiz_0 r_chaos_quiz_0]; % 
R_test = [R_test r_chaos_test_1 r_chaos_test_0 r_chaos_test_0]; 
            % the last column is repeated, it will be used to store the
            %   predictions of DOC method
R_test1 = R_test(ind_test1_test,:);
R_test2 = R_test(ind_test2_test,:);

save(chaosFile,'R_test','R_quiz','-append');
fprintf('* Finished to predict the ratings of the each group of sheep.\n');

% 5. Calculate the prediction accuracies on each of White, Gray, Black, Blue
% and Whole sheep;      
nSet = 3; % three set: Quiz, Test_1, Test_2
nModel = size(R_quiz,2); 
MAE = zeros(nModel,len_group_v_quiz,nSet);
  % each row store the MAE for a model in White, Gray, Black, Blue and Whole sheep;         
  % Page 1: for Quiz Set;
  % Page 2: for Test_1 Set;
  % Page 3: for Test_2 Set;
acc1 = zeros(nModel,len_group_v_quiz,nSet);
acc2 = zeros(nModel,len_group_v_quiz,nSet);
BadEntryRate = zeros(nModel,len_group_v_quiz,nSet);
ErrorRate = zeros(nModel,len_group_v_quiz,nSet);
RMSE = zeros(nModel,len_group_v_quiz,nSet);
%sets = {'quiz','test'};
sets = {'quiz','test1','test2'};
for i_set = 1:length(sets)
    group_v = eval(sprintf('group_v_%s',sets{i_set}));
    R = eval(sprintf('R_%s',sets{i_set}));
    rr = eval(sprintf('rr_%s',sets{i_set}));
    for i_model = 1:nModel
        for i_group = 1:len_group_v_quiz % len_group_v_quiz == len_group_v_test
            id_v = eval(group_v{i_group});
            if ~isempty(id_v)
                [MAE(i_model,i_group,i_set),acc1(i_model,i_group,i_set),acc2(i_model,i_group,i_set),...
                BadEntryRate(i_model,i_group,i_set),ErrorRate(i_model,i_group,i_set),RMSE(i_model,i_group,i_set)] ...
                   =  indicator_precision([],[],R(id_v,i_model),rr(id_v),0); 
            else % id_v is empty
                MAE(i_model,i_group,i_set) =0;
                acc1(i_model,i_group,i_set)=0;
                acc2(i_model,i_group,i_set)=0;
                BadEntryRate(i_model,i_group,i_set)=0;
                ErrorRate(i_model,i_group,i_set)=0;
                RMSE(i_model,i_group,i_set) =0;                   
            end
        end
    end
end

% 6. calculate the DOC prediction based on BC predictions on test1 (R_test1)
i_set = 2; % set of test1
i0_model = size(R,2)-1; % -1: the last column is preseved for DOC predictions
obj_model = size(R,2); % the index of the last column 
for i_group = 1:len_group_v_test            
    % initialize j_model, accj, acc0
    j_model = i0_model; % compared with the prediction of BC method 
    acc0 = [MAE(i0_model,i_group,i_set),RMSE(i0_model,i_group,i_set)];
    %accj = [MAE(j_model,i_group,i_set),RMSE(j_model,i_group,i_set)];
    for i_model = 1:nModel        
        acc1 = [MAE(i_model,i_group,i_set),RMSE(i_model,i_group,i_set)];
        %if eval_better(acc1,acc0,1)
        if eval_better(acc1,acc0,3)
            % find  a model is better than the current one on a given group of sheep
            %if eval_better(acc1,accj,2) % accj: current ``best'' accuracy under rule 2
                j_model = i_model;                     
            %    accj = acc1;
                acc0 = acc1; % acc0: current ``best'' accuracy under rule 3
            %end
        end
    end
    % assign the best individual predictions as DOC predictions
    if j_model~=i0_model      
        for i_set = 1:length(sets)
            group_v = eval(sprintf('group_v_%s',sets{i_set}));
            id_v = eval(group_v{i_group});
            eval(sprintf('R_%s(id_v,obj_model)=R_%s(id_v,j_model);',sets{i_set},sets{i_set}));
        end        
    end
end

% calculate the prediction of the DOC predictions
sets = {'quiz','test1','test2'};
i_model = obj_model;
for i_set = 1:length(sets)
    group_v = eval(sprintf('group_v_%s',sets{i_set}));
    R = eval(sprintf('R_%s',sets{i_set}));
    rr = eval(sprintf('rr_%s',sets{i_set}));    
    for i_group = 1:len_group_v_quiz % len_group_v_quiz == len_group_v_test
        id_v = eval(group_v{i_group});
        if ~isempty(id_v)
            [MAE(i_model,i_group,i_set),acc1(i_model,i_group,i_set),acc2(i_model,i_group,i_set),...
                BadEntryRate(i_model,i_group,i_set),ErrorRate(i_model,i_group,i_set),RMSE(i_model,i_group,i_set)] ...
                =  indicator_precision([],[],R(id_v,i_model),rr(id_v),0);
        else % id_v is empty
            MAE(i_model,i_group,i_set) =0;
            acc1(i_model,i_group,i_set)=0;
            acc2(i_model,i_group,i_set)=0;
            BadEntryRate(i_model,i_group,i_set)=0;
            ErrorRate(i_model,i_group,i_set)=0;
            RMSE(i_model,i_group,i_set) =0;
        end
    end    
end


% MAE and RMSE of chaos_clip (on test set)
MAE_opt = MAE(obj_model,len_group_v_quiz,nSet); % best MAE on test2
RMSE_opt = RMSE(obj_model,len_group_v_quiz,nSet); % best RMSE on test2

save(chaosFile,'MAE','acc1','acc2','BadEntryRate','ErrorRate','RMSE','-append');
save(chaosFile,'MAE_opt','RMSE_opt','-append');
fprintf('* Finished to calculate the prediction accuracies on each of these four groups.\n');

end

function v = eval_better(accA,accB,flag)
% determine whether an accuracy vector, accA, is better than the other one,
% accB
% Inputs:
%  accA, acc0: two vectors, each one has two scalars;
%  flag: 1 or 2:
%    1: indicating Rule 1:
%       accA is better than accB if 
%           accA(1) < accB(1)    and    accA(2) < accB(2);
%    2: indicating Rule 2: 
%       accA is better than accB if 
%      lambda*accA(1)+(1-lambda)*accA(2) < lambda*accB(1)+(1-lambda)*accB(2) 
%      where 0<=lambda<=1 is a constant;
%    3: Rule 3:
%       accA is better than accB if 
%             (accB(1) - accA(1))/accA(1) >= evident_level
%       where evident_level>0 is a given constant;
%       This rule only compares the first accuracy.
%    3: Rule 4:
%       accA is better than accB if 
%             (accB(2) - accA(2))/accA(2) >= evident_level
%       where evident_level>0 is a given constant;
%       This rule only compares the second accuracy.

lambda = 0.5;
evident_level = .003;

if flag==1
    v = accA(1)<accB(1) & accA(2)<accB(2);
elseif  flag ==2
    v = lambda*accA(1)+(1-lambda)*accA(2) < lambda*accB(1)+(1-lambda)*accB(2); 
elseif flag==3
    v = (accB(1)-accA(1))/accA(1)>=evident_level;
else %flag==4
    v = (accB(2)-accA(2))/accA(2)>=evident_level;
end

end

