function [sol, fval, inform] = SolveGeneralLasso(A,y,w,lambda,kappa,arg)
% solve generalized Lasso problem
% Inputs
%	A           Either an explicit nxN matrix, with rank(A) = min(n,m) 
%               by assumption, or a string containing the name of a 
%               function implementing an implicit m atrix (see below for 
%               details on the format of the function).
%	y           vector of length m.
%   w           the scale factors. 
%   lambda      If specified (and > 0), the algorithm terminates when the
%                 Lagrange multiplier<=lambda. 
%   kappa       a positive scalar, constaint: norm(w.*x,inf)<=kappa 
%   arg         Optional, a structue with following fields:
%    arg.maxIters   maximum number of Lars iterations to perform. If not
%                     specified, runs to stopping condition (default)
%    arg.resStop    If specified (and > 0), the algorithm terminates when the
%                     L2 norm of the residual <= resStop. 
%    arg.verbose    1 to print out detailed progress at each iteration, 0 for
%                     no output (default)
%    arg.OptTol     Error tolerance, default 1e-5
%    arg.maxActInd  maximum number of active indices, default value 0, if
%                    this parameter is set, then the algorithm terminates
%                    when the number of nonzero elements of the solution
%                    reach the given bound;
%                    WARNING: if this argument is set, the function may
%                    terminate without obtaining an optimal solution.
%    arg.parCheck   the level to check parameters, 0: no check; 1: check
%                    the size of inputs; 2: check the size of inputs and
%                    check identical or oppposite columns of A, default value 2;
% Outputs
%   fval           optimal function value             
%	sol            solution of the generalized Lasso problem;
%   inform         a structure indicating iteration information  
%     .exitFlag       <=0: failed to find the optimal solution;
%                    >=1: succeed to find the optimal solution;
%                       More particularly,
%                      2: succeed to find the optimal solution, both elements are active
%                      1: succeed to find the optimal solution;
%                      0: failed to find the optimal solution;
%                     -1: exceeding maximum iterations          
%                     -2; residual y-Ax reach given bound
%                     -3; the number of active indices reach given bound
%	  .iter           total number of steps taken;
%     .activeIndices  array of indices showing the active indices entering the
%                  solution set;
%     .activationHist array of indices showing elements entering and 
%                    leaving the active set II;
%                    e.g.  [6 7 12 8 -6 -0.07 0.12 ] indicates that the
%                     changes of indices set
%                      iter 1.  6: III --> II
%                      iter 2.  7: III --> II
%                      iter 3.  12: III --> II
%                      iter 4.  8: III --> II
%                      iter 5.  6: II --> III
%                      iter 6.  7: II --> I
%                      iter 7.  12: I --> II
%                   The rule is that positive number means "entering into
%                   II"; negative number means "moving out from II";
%                   Integer indicates the interchange between II and III,
%                   while a decimal indicates the interchange between I and
%                   II;
%     .residual     value of residual |A*x-y| at termination;
%     .lambda       value of lambda at termination;

% Description
%   The original algorithm is implemented, as well as the lasso modification, 
%   which solves 
%      min lambda*||x||_1 + 1/2|| y - Ax ||_2^2
%   for all lambda >= 0, using a path following method with parameter lambda.
%   This function  SolveGeneralLasso() solves
%       min     lambda*||x||_1 + 1/2||Ax-y||_2^2
%       s.t.    w_i*|x_i| <= kappa,
%   where w_i>=0 is scale factor, i = 1,...,m;
%   lambda>0, kappa>0 are given parameters. 

% References
%   [1] B. Efron, T. Hastie, II. Johnstone and R. Tibshirani, 
%   "Least Angle Regression", Annals of Statistics, 32, 407-499, 2004
%   [2] D. L. Donoho, Y. Tsaig. Fast solution of l1-norm minimization problems 
%   when the solution may be sparse. IEEE Transacions on Information
%   Theory, 54(11): 4789--4812, 2008.
%
%   we have refered the code of of Tssaig, Stanford University ----

    function assign_inform()
        % assign value to the struct inform
        inform = struct('exitFlag',exitFlag,'iter',iter, 'activeIndices',activeIndices,...
            'activationHist',activationHist,'residual',norm(res),'lambda',lambda);
    end
% Initialization
exitFlag = -1;
sol = 0;
fval = 0;
iter = 0;
activeIndices = [];
activationHist =[];
res = y; % residual = norm(A*x-y) with x = 0 at inialization;
inform = [];

% set parameters
if nargin <=5
    arg = struct();
end
if ~isfield(arg,'verbose') || isempty(arg.verbose) 
    arg.verbose = 0;
end
if ~isfield(arg,'resStop') || isempty(arg.resStop) || arg.resStop<0
    arg.resStop = 0;
end
if ~isfield(arg,'maxIters') || isempty(arg.maxIters) || arg.maxIters<0
    arg.maxIters = 3*size(A,2); 
end
if ~isfield(arg,'maxActInd')|| isempty(arg.maxActInd) || arg.maxActInd<0
    arg.maxActInd = 0;
end
if ~isfield(arg, 'parCheck') || isempty(arg.parCheck) 
    arg.parCheck = 2;
end

verbose = arg.verbose;
resStop = arg.resStop;
maxIters = arg.maxIters;
maxActInd = arg.maxActInd;
parCheck = arg.parCheck;

% check efficiency of input parameters
[m n] = size(A);
if parCheck>=1    
    if m ~= length(y)
        warning(1,'the row number of A should be identical with the length of y.');
        return
    end
    if n~=length(w)
        warning(1,'the column number of A should be identical with the length of w.');
        return
    end
    if issparse(A)
        A = full(A); % current program not support sparse matrix A
        warning(2,'current program not support sparse matrix A. It has been convert to full matrix.');
    end
    if isempty(lambda)||lambda<=0
        warning(3,'The argument lambda should be a given positive scalar.');
        lambda = 0.1;
        warning(3,'lambda has been reset as lambda=%.2f\n',lambda);
    end
    if isempty(kappa)||kappa<=0
        warning(3,'The argument kappa should be a given positive scalar.');
        kappa = 0.5;
        warning(3,'kappa has been reset as kappa=%.2f\n',kappa);
    end
end

zeroTol = 1e-12; 
lambdaStop = lambda; 
    % we copy the input argument lambda as lambdaStop, the variable lambda
    % is reset in each of the iterations
    
% check the identical or opposite column vectores of A 
identicalIndices = [];
if parCheck>=2
    [A1 ia ia1] = unique(A','rows');
      % deal with the opposite column vectors
    [B1 iia iia_minus] =intersect(A(:,ia)',-A(:,ia)','rows');     
    identical_cell = []; % store identical or opposite column indices of A  
    if length(ia)~=length(ia1) || ~isempty(B1)
        ia1 = replace_real(ia1,1:length(ia),ia);        
        if ~isempty(B1)
            % note that the two columns A(:,ia(iia(ii))), and A(:,ia(iia_minus(ii))) are opposite to each other;
            %  e.g. ia(iia)      = [ 4 5 9 10]';
            %       ia(iia_minus)= [ 5 4 10 9]';
            %  which means that the columns 4,5 of matrix A are opposite column vectors and
            %   the columns 9, 10 of matrix A are opposite column vectors
            
              % find the unique pairs of the indices of ia(iia) and ia(iia_minus)
              %  for the example above:  
              %  e.g. ia(iia)      = [ 4 5 9 10]'   ---> [4 9]'
              %       ia(iia_minus)= [ 5 4 10 9]'   ---> [5 10]'
              % (4,5) and (5,4) are  pairs with the same meaning              
            c = phi_sys(ia(iia),ia(iia_minus)); 
            [c1 ic]  = unique(c);
            
            ia1 = replace_real(ia1,ia(iia_minus(ic)),ia(iia(ic)));
            ia(iia_minus(ic)) = [];	
        end
        if length(ia)~=length(ia1) % there exists identical or opposite columns
            k = 1;
            for ii = 1:length(ia)
                ind = find(ia1==ia(ii));
                if nnz(ind)>1
                    identical_cell{k} = ind;
                    k = k+1;
                end
            end        	
        end
    end
  % reset w
    identicalIndices = []; 
    if ~isempty(identical_cell)
        w_copy = w;
        for ii = 1:length(identical_cell)
            K = identical_cell{ii};
            [w_min id_w_min] = min(w(K));
            if w_min>0
                w(K(id_w_min)) = 1/sum(1./w(K));  % reset w(K)
            end
            % store the indices of collinear columns (exclude K(id_w_min) )
            K(id_w_min) = []; % exlude the index K(id_w_min)
            identicalIndices = [identicalIndices; K];
        end        
    end
end

% parameters for linsolve function
opts.UT = true; 
opts_tr.UT = true; opts_tr.TRANSA = true;
nbit = power(10,ceil(log10(n))); % number of digits of n

x = zeros(n,1);
sol = []; 
res = y;
iter = 0;
exitFlag = 0; % not find the solution
collinearIndices = [];
fval = 0.5*(y'*y);

% First vector to enter the active set is the one with maximum correlation
corr = A'*y;             
lambda = max(abs(corr));
newIndices = find(abs(abs(corr)-lambda)<zeroTol);    
newIndices = columnVec(setdiff(newIndices,identicalIndices)); 
    % exclude the identicalIndices
    % columnVec() ensure a column vector

% Check stopping conditions
if lambdaStop>0 && lambda<=lambdaStop || resStop>0 && norm(res,1)<resStop    
    activationHist = [];
    exitFlag = 1;
    assign_inform();
    return;
end
   
% Initialize index sets I, II, III and Cholesky factor of A_II
I = []; II = []; R_II = [];
for j = 1:length(newIndices)
    iter = iter+1;
    [R_II, flag] = updateChol(R_II, newIndices(j));
    if ~flag
        II = [II; newIndices(j)]; % ensure that II is column vector
        if verbose>1            
            fprintf('Iteration %d: variable %d\t III-->II.\n', iter, II(j));
        end
    else
        collinearIndices = [collinearIndices; newIndices(j)];        
    end
end
activationHist = II;
activeIndices = II;
III = 1:n;
III([newIndices; identicalIndices]) = []; % donot forget to exclude the identicalIndices
III = columnVec(III);
	% inactive set, columnVec() ensure III is a column vector
%%%%
    num_col = 0;
    flag_data_saved = 0;
%%%%                
gammaMin = 0;
while  1    % begin iteration
    flag_empty_II = isempty(II); 
    if flag_empty_II % if II is empty, choose variables from III to II
        ind_max_III = [];
        if ~isempty(III)
            [lambda ind_max_III]= max(abs(corr(III))); 
                % ind_max_III is used in updating the indices
        else  % both II and III is empty            
            if verbose>1
                fprintf('the iteration terminates since both II and III are empty.\n');
            end
            exitFlag = 2; % succeed to find the solution, both II and III are empty
            assign_inform();
            break;
        end
    else 
        lambda = abs(corr(II(1)));
    end    
    if verbose==2
        fwritef(1,'I',I','%d\t','II',II','%d\t','III',III','%d\t','collinearIndices',collinearIndices','%d\t',...
            'identicalIndices',identicalIndices','%d\t');        
    end
    if verbose==3
        fwritef(1,'corr',corr','%.2f\t','x',x','%.3f\t');
        fwritef(1,'gammaMin',gammaMin,'');
    end
    if verbose==1
        fprintf('lambda = %11.8f, |II| = %d, normres = %3.2f\n', lambda, length(II), norm(res));
    end
    
    % 0. check stopping condition    
    if lambdaStop>0 && lambda<=lambdaStop
        exitFlag = 1; 
        assign_inform();
        break;
    end
    if maxIters>0 && iter>=maxIters
        exitFlag = -1; % not succeed to find the solution, exceeding maximum iterations
        assign_inform();
        break;
    end
    if resStop>0 && norm(res)<=resStop 
        exitFlag = -2; % not succeed to find the solution, residual y-Ax reach given bound
        assign_inform();
        break;
    end
    if  maxActInd>0 && length(activeIndices)>=maxActInd
        exitFlag = -3; % not succeed to find the solution, the number of active indices reach given bound
        assign_inform();
        break;
    end  
    
    if ~flag_empty_II
        % 1. compute descent direction - Equiangular vector
            % solve the equation (A_II'*A_II)dx_II = sgn(corr_II)
        z = linsolve(R_II,sign(corr(II)),opts_tr);
        dx_II = linsolve(R_II,z,opts);   
        if verbose==3
            fwritef(1,'dx_II',dx_II','%.2f\t');
        end
        % 2. compute the step size
        v = A(:,II)*dx_II;
        ATv = A'*v;
          % 2(1) find first active vector to violate sign constraint           
        II_ind = abs(dx_II)>zeroTol;
        IIa = II(II_ind); 
        zc = -x(IIa)./dx_II(II_ind);        
        zc(zc<zeroTol) = lambda;
        gamma_minus = min(zc);
        indices_minus = [];
        if abs(lambda-gamma_minus)>zeroTol %lambda~=gamma_minus 
            indices_minus = IIa(abs(zc-gamma_minus)<zeroTol);
        end        
           % 2(2) find first inactive vector to enter the active set
        gamma_plus = lambda; % 0 <= gamma_plus <= lambda-lambdaStop
        indices_plus = [];    
        if ~isempty(III)             
            i_corr_lambda = abs(lambda-corr(III))<zeroTol; % whether corr(III) == lambda            
            IIIc = III(i_corr_lambda & sign(corr(III)).*sign(ATv(III))==-1);            
            if ~isempty(IIIc) % stuck case III occurs, some calculation errors may occur, 
                              %     since this is imposible case theoratically.  
                gamma_plus = 0;
                indices_plus = IIIc;
                if verbose
                    fprintf('warning: iteration %d:\t stuck case III occurs.\n',iter);
                end
                %%%%
                if verbose==5
                %fileDat = sprintf('colData%s.mat',datestr(now,30));
                %save(fileDat,'A','y','w','lambdaStop','kappa','arg');                
                fwritef(1,'',IIIc','%d\t','ATv(IIIc)',ATv(IIIc)','%.6f\t',...
                    'corr(IIIc)',corr(IIIc)','%.6f\t');
                end
                %%%
            else
                IIIa = III(1-ATv(III)>zeroTol & ~i_corr_lambda);
                IIIb = III(1+ATv(III)>zeroTol & ~i_corr_lambda);
                gammaArr1 = []; gammaArr2 = [];
                isempty_IIIa = isempty(IIIa);
                isempty_IIIb = isempty(IIIb);
                if ~isempty_IIIa
                    gammaArr1 = (lambda-corr(IIIa))./(1-ATv(IIIa));
                end
                if ~isempty_IIIb
                    gammaArr2 = (lambda+corr(IIIb))./(1+ATv(IIIb));
                end
                if ~isempty_IIIa || ~isempty_IIIb                    
                    gammaArr_plus = [gammaArr1; gammaArr2];
                    gammaArr_plus(gammaArr_plus<0) = lambda;
                        % make sure each element is nonnegative
                    gamma_plus = min(gammaArr_plus);
                    if ~isempty_IIIa && ~isempty_IIIb
                        indices_plus = [IIIa(abs(gammaArr1-gamma_plus)<zeroTol); ...
                                    IIIb(abs(gammaArr2-gamma_plus)<zeroTol)]; 
                    elseif ~isempty_IIIa % ~isempty_IIIb ==0
                        indices_plus = IIIa(abs(gammaArr1-gamma_plus)<zeroTol);
                    else
                        indices_plus = IIIb(abs(gammaArr2-gamma_plus)<zeroTol); 
                    end
                end
            end
        end
          % 2(3) find indices reach the appended constraint w_i|x_i + gamma d_i|<= kappa
        gamma_minus_minus = lambda;
        indices_minus_minus = [];    
        II_ind = abs(dx_II)>zeroTol & w(II)>0 & (sign(dx_II).*sign(x(II))>=0);
        II_a = II(II_ind);  
        if ~isempty(II_a)            
            gammaArr = [(kappa./w(II_a)-x(II_a))./dx_II(II_ind) ...
                        (-kappa./w(II_a)-x(II_a))./dx_II(II_ind) ]; % gammaArr is a matrix with 2 columns
            gammaArr(gammaArr<0) = lambda;
            gammaArr_minus_minus = min(gammaArr,[],2);
            gamma_minus_minus = min(gammaArr_minus_minus);
            indices_minus_minus = II_a(abs(gamma_minus_minus-gammaArr_minus_minus)<zeroTol);
        end    
          % 2(4) find indices of I with magnitudes of the residual corerelation
          %  decreasing to lambda-gamma
        gamma_plus_plus = lambda; % 0<=gamma_plus_plus<=lambda
        indices_plus_plus = [];            
        I1=I( corr(I)>0 & ATv(I)-1>zeroTol); 
        I2=I( corr(I)<0 & ATv(I)+1<zeroTol); 
        isempty_I1 = isempty(I1);
        isempty_I2 = isempty(I2);
        if ~isempty_I1|| ~isempty_I2            
            gammaArr1 = [];
            gammaArr2 =[];
            if ~isempty_I1
                gammaArr1 = (lambda-corr(I1))./(1-ATv(I1));
            end
            if ~isempty_I2
                gammaArr2 = (lambda+corr(I2))./(1+ATv(I2));	
            end
            gammaArr = [gammaArr1; gammaArr2]; 
            gammaArr(gammaArr<0) = lambda; % ensure each stepsize is nonnegative                                                     
            [gamma_plus_plus] = min(gammaArr);
            Imin = find(abs(gammaArr-gamma_plus_plus)<zeroTol);
            if ~isempty_I1 && ~isempty_I2
                n1 = length(I1);
                indices_plus_plus = [I1(Imin(Imin<=n1)); I2(Imin(Imin>n1)-n1)];                
                        % both I1 and I2 are column vectors
            elseif ~isempty_I1 % ~isempty_I2==0
                indices_plus_plus = I1(Imin);                
            else % ~isempty_I1==0, ~isempty_I2==1
                indices_plus_plus = I2(Imin);                
            end           
        end  

        % 3. compute the next step
        gammaMin= min([gamma_plus,gamma_minus,gamma_plus_plus,gamma_minus_minus]);        
        if verbose==4
            fwritef(1,'gammaMin',gammaMin,'%.6f');
            fwritef(1,'gamma_plus',gamma_plus,'%.6f','gamma_minus',gamma_minus,'%.6f',...
             'gamma_plus_plus',gamma_plus_plus,'%.6f','gamma_minus_minus',gamma_minus_minus','%.6f');
        end        
        if gammaMin>lambda-lambdaStop 
            gammaMin=lambda-lambdaStop; 
                % the stepsize should be located in the interval [0,lambda-lambdaStop]
        end
        x(II) = x(II) + gammaMin*dx_II;
        res = res - gammaMin*v;
        corr = corr - gammaMin*ATv;
    end % end if ~flag_empty_II
    
    % Add new indices to active set II
    if ~flag_empty_II
        act_gamma_p = gamma_plus<=gammaMin;
        act_gamma_p_p =  gamma_plus_plus<=gammaMin;
    else
        act_gamma_p = 0;
        act_gamma_p_p = 0; 
    end
    if flag_empty_II || act_gamma_p || act_gamma_p_p
        if flag_empty_II
            newIndices = III(ind_max_III);
        elseif act_gamma_p && act_gamma_p_p 
            newIndices = [indices_plus; indices_plus_plus];             
        elseif act_gamma_p % act_gamma_p_p==0
            newIndices = indices_plus;
        else % act_gamma_p_p==1, act_gamma_p ==0  
            newIndices = indices_plus_plus;
        end        
        
        if flag_empty_II
            len_indices_plus = length(ind_max_III);  % len_indices_plus: number of indices of set III to be added to set II
        elseif act_gamma_p
            len_indices_plus = length(indices_plus);
        else
            len_indices_plus = 0;
        end            
        for j = 1:length(newIndices)
            iter = iter+1;
    	    ind = newIndices(j);  
            ind_in_III = j<=len_indices_plus; % whether  index IND belongs to set III
            % Update the Cholesky factorization of A_I
            [R_II, flag] = updateChol(R_II,ind);
            % Check for collinearity
            if flag % collinear case
                % move the index IND from I or III to collinearIndices
                collinearIndices = [collinearIndices; ind];
                if ind_in_III
                    III(III==ind) = [];
                else % index IND belongs to set I
                    I(I==ind) = [];
                end
                if verbose
                    fprintf('Iteration %d: Variable %d is collinear\n', iter, ind);                    
                end         
                %%%%
                if verbose==4
                    num_col = num_col + 1;
                    if num_col > 1 && ~flag_data_saved
                        fileDat = sprintf('colData%s.mat',datestr(now,30));
                        save(fileDat,'A','y','w','lambdaStop','kappa','arg');
                        flag_data_saved = 1;
                    end
                end
                %%%%
            else                
	      	   % update the index sets I, II, and III
                II = [II; ind];
                if ind_in_III
                    % move the index IND from  III to II
                    III(III==ind) = [];
                    activationHist = [activationHist; ind];
                    activeIndices = [activeIndices; ind];
                else % index IND belongs to set I
                    % move the index IND from  I to II
                    I(I==ind) = [];
                    activationHist = [activationHist; ind/nbit];
                        % the indices in I are also activeIndices, ind has
                        %  exist in actIndices, needs not to add into activeIndices
                end
                if verbose>1
                    if flag_empty_II || act_gamma_p
                        fprintf('Iteration %d: variable %d\t III --> II.\n', iter,ind);
                    else 
                        fprintf('Iteration %d: variable %d\t I --> II.\n', iter,ind);
                    end
                end 
            end
        end
    end

    % Remove violating indices from active set    
    if ~flag_empty_II
        act_gamma_m = gamma_minus<=gammaMin;
        act_gamma_m_m = gamma_minus_minus<=gammaMin;
    else
        act_gamma_m = 0;
        act_gamma_m_m = 0; 
    end    
    if ~flag_empty_II && (act_gamma_m || act_gamma_m_m)
        if act_gamma_m && act_gamma_m_m
            indices_remove = [indices_minus; indices_minus_minus];
            activationHist = [activationHist; -indices_minus; -indices_minus_minus/nbit];
        elseif act_gamma_m
            indices_remove = indices_minus;
            activationHist = [activationHist; -indices_minus];
        else % act_gamma_m_m==1;
            indices_remove = indices_minus_minus;
            activationHist = [activationHist; -indices_minus_minus/nbit];
        end
        for j = 1:length(indices_remove)
            iter = iter+1;
            col = find(II==indices_remove(j));
            if verbose>1
                if act_gamma_m
                    fprintf('Iteration %d: variable %d\t II --> III.\n', iter, indices_remove(j));
                else % act_gamma_m_m
                    fprintf('Iteration %d: variable %d\t II --> I.\n', iter, indices_remove(j));
                end
            end
            % Downdate the Cholesky factorization R_II
            R_II = downdateChol(R_II,col);
            II(col) = [];             
        end
	    % update the index sets I, III, activeIndices, collinearIndices
        if act_gamma_m_m
            I = [I; indices_minus_minus];
                % the indices in I are defined as active indices, the 
                %   indices in INDICES_MINUS_MINUS needs not to remove from 
                %   ACTIVEINDICES
        end
        if act_gamma_m 
            III = [III; indices_minus];   
                % keep in mind that all I, II III are column vectors
            activeIndices = replace_real(activeIndices,indices_minus,[]);   
            x(indices_minus) = 0;  % to avoid numerical errors    
        end
        if ~isempty(collinearIndices)
            ind = abs(x(collinearIndices))>=zeroTol;
            I=[I; collinearIndices(ind)];
            III = [III; collinearIndices(~ind)];
            collinearIndices = []; % reset collinear set
        end        
    end
end

% reset the elements of the solution x with indices corresponding to
%  identical/opposite column vectors of A
if parCheck>=2 && ~isempty(identical_cell)
    for ii = 1:length(identical_cell)
        K = identical_cell{ii};
        [wK id_wK] = sort(w_copy(K));
        ind0 = K(id_wK(1));        
        xx0 = x(ind0); 
        xx0_mag = abs(xx0);
	    if wK(1)>0 && wK(1)*xx0_mag>kappa 
            for jj=1:length(wK)
                ind = K(id_wK(jj));
                if wK(jj)*xx0_mag<=kappa
                    x(ind) = sign(xx0) * ((norm(sum(A(:,[ind0 ind]),2),1)>zeroTol)-0.5)*2 * xx0_mag;
                    break;
                end                
                x(ind) = sign(xx0) * ((norm(sum(A(:,[ind0 ind]),2),1)>zeroTol)-0.5)*2 * kappa/wK(jj); % reset x(ind)
                      % ((norm(sum(A(:,[ind0 ind]),2),1)>zeroTol)-0.5)*2: -1:  if the two column vectors 
                      %    A(:,ind0) and A(:,ind) are opposite, 1: otherwise (identical two vectors); 
                xx0_mag = xx0_mag-kappa/wK(jj);                
            end
        end
    end
end
sol = x;
fval = funcVal(x);

% ------ embeded functions of SolveGeneralLasso() ----

function [R, flag] = updateChol(R,newIndex)
% updateChol: Updates the Cholesky factor R of the matrix 
% A(:,II)'*A(:,II) by adding A(:,newIndex)
% If the candidate column is in the span of the existing 
% active set, R is not updated, and flag is set to 1.
flag = 0;
newVec = A(:,newIndex);
if isempty(II)
    R = sqrt(sum(newVec.^2));
else
    p = linsolve(R,A(:,II)'*A(:,newIndex),opts_tr);
    q = sum(newVec.^2) - sum(p.^2);
    if (q <= zeroTol) % A(:,newIndex) is in the span {A(:,i)}, i \in II
        flag = 1;
    else
        R = [R p; zeros(1, size(R,2)) sqrt(q)];
    end
end
end % end the function updateChol()

function R = downdateChol(R, j)
% downdateChol: `Downdates' the cholesky factor R by removing the 
% column indexed by j.

% Remove the j-th column
if length(R)==1
    R = []; 
    return;
end
if j==length(R) % remove the last indices
    R(:,j) = [];
    R(j,:) = [];
    return
end
R(:,j) = [];
len = size(R,2);
% R now has nonzeros below the diagonal in columns j through m;
% We use plane rotations to zero the 'violating nonzeros'.
for kk = j:len
    p = kk:kk+1;
    [G,R(p,kk)] = planerot(R(p,kk));
    if kk<len
        R(p,kk+1:len) = G*R(p,kk+1:len);
    end
end
% Remove last row of zeros from R
R = R(1:len,:);
end % end of the function downdateChol()

function fval = funcVal(x)
    %fval = 0.5*sum(square(A*x-y)) + lambdaStop*norm(x,1);
    fval = 0.5*sum((A*x-y).^2) + lambdaStop*norm(x,1);
end

end % end of the function SolveGeneralLasso()

%------------ subfunctions ---------------

function y = columnVec(x)
% transfrom a vector to column vector
  if size(x,2)~=1
    y = x';
  else
    y = x;
  end    
end

function B = replace_real(A,sub,obj)
% replace the values of the matrix A
% Inputs:
%  A: a real matrix
%  sub, obj:   elements of sub are different with each other;
%   1) if sub and obj are two vectors with the same length: the function replace the elements 
%       of sub(i) of A into obj(i), for all i;
%   2) if obj is a scalar or empty: the function replace the elements of
%       sub(i) of A into obj, for all i;
% Outputs:
%  B: the matrix after the values replaced
B = A;
if length(sub) == length(obj)
    for ii=1:length(sub)
        B(A==sub(ii)) = obj(ii);	
    end
elseif length(obj)<=1
    B(ismember(B,sub)) = obj;
else
    warning(1,'the size of sub or obj is illegal.');
end
end

function C = phi_sys(A,B)
% For scalar case,
%  phi_sys: Z+ * Z+  --> R+ satisfies: for any positive integer a,b
%  1) phi_sys(a,b) == phi_sys(b,a), 
%  2) phi_sys(a1,b1) ~= phi_sys(a2,b2), if (a1,b1)~=(a2,b2) and
%    (a1,b1)~=(b2,a2)
% For vector or matrix A and B with the same size, the function operates on
%   corresponding  elements of A and B;

% Inputs:
%  A,B: two vectors or matrices with the same size, each element is positve integer
% Outputs:
%  C: a vector or matrix with the same size as A and B
C=A.*A+B.*B+min(A,B)./max(A,B);
end
