function h_MAE_line = draw_index_dist(index_v, varargin)
% draw the distribution based on given index index_v
% Inputs:
%   index_v: column vector containing the values of a certain index,
%   varargin: a cell array containing arguments Name afterwards arguments
%       Value pair by pair, such as {'num_entry',[3 7 8], 'indexName','coherece};
%   Optional argument names:
%      'num_entry': a column vector with the same length as INDEX_V,
%        indicating number of entries located in each interval;
%      'MAE_vec': a column vector with the same length as INDEX_V, MAE values
%      'ErrorNum_vec': a column vector with the same length as INDEX_V,
%        numbers of Error entries
%      'index_range': a 2*1 vector, range to draw the index
%      'indexName': a string, name of the index, default: 'unrelatedness'
%      'MAE_whole': a scalar, whole MAE
%      'MAE_white': a scalar, MAE baseline for white sheep
%      'labelFontSize': positive integer, font size of  xlabel and ylabel
%      'lineColor' : color of lines 
%      'lineStyle' : line style: {-} | -- | : | -. | none
% Outputs:
%   h_MAE_line: handle for index--MAE line, [] if no index-MAE line is
%   drawn
% Usage example: h_line_v = draw_index_dist(relatedness_dist,'MAE_vec',mae_vev,...
%                 'index_range',[1,5],'indexName','Entropy','modelName','nNbr_item');
             
% 0. Initiation
argument_cell = {'num_entry','MAE_vec','ErrorNum_vec','index_range','MAE_whole',...
    'MAE_white','indexName', 'labelFontSize','lineColor','lineStyle','colorBar','lineWidth'};
[num_entry,  MAE_vec, ErrorNum_vec,index_range,MAE_whole,MAE_white,indexName, ...
    labelFontSize,lineColor,lineStyle,colorBar,lineWidth] = getArgument(varargin,argument_cell);

flag_ishold = ishold;
h_MAE_line = [];

if isempty(labelFontSize)
    labelFontSize = 11; % default font size of xlabel and ylabel 
end

    % 1. plot bar graph: index_v -- num_entry
flag_barPlot = 1; % indicate whether plot bars    
h_bar = [];
if ~isempty(ErrorNum_vec)&&~isempty(num_entry)
    h_bar = bar(index_v, [ErrorNum_vec num_entry-ErrorNum_vec],'Stack');
    legend('#error entries','#entries');
elseif  ~isempty(num_entry)   
    h_bar = bar(index_v, num_entry);
else
    flag_barPlot = 0; % no bars are plot
end

if ~isempty(h_bar) && ~isempty(colorBar)  
    % set color for faces of bars
    for ii=1:length(colorBar)
        set(h_bar(ii),'FaceColor',colorBar{ii});
    end
end


    % 1.2 set XLim and YLabel
flag_plot_MAE_line = ~isempty(MAE_vec);
%if flag_barPlot 
    h1 = gca;
    if ~isempty(index_range)
        set(h1,'XLim',index_range);
    end
    set(h1,'Layer','top');    
    ylabel('number of tested entries','fontsize',labelFontSize);    
%end

    % 2. plot MAE line: index_v -- MAE
    % 2.1 set the currenct axes h2
if flag_plot_MAE_line
    if flag_barPlot 
        h2 = axes('Position',get(h1,'Position'));
    else
        h2 = gca;
    end
        % 2.2 set the hold state
    if flag_barPlot 
        hold on
    end
        % 2.3 plot the index -- MAE line    
        % randomly generate a color
    if isempty(lineColor)
        lineColor = rand(3,1);
    end
    if ~isempty(MAE_vec)     
        h_MAE_line = plot(h2,index_v,MAE_vec,'linewidth',lineWidth,'Color',lineColor,...
            'LineStyle',lineStyle);
        % set the lengend
    end

        % 2.4 plot MAE_whole and MAE_white
    if ~isempty(MAE_whole)
        plot(h2,index_range,MAE_whole*[1,1],'-.','Color',lineColor); 
    end
    if ~isempty(MAE_white)
        plot(h2,index_range,MAE_white*[1,1],':','Color',lineColor); 
    end

       % 2.5 set YTick
    if ~isempty(MAE_whole)||~isempty(MAE_white)
        y_tick = get(gca,'YTick');
        % exclude the ticks near MAE_whole and MAE_white
        radius = ((max(y_tick)-min(y_tick))/length(y_tick))/4;  
        if ~isempty(MAE_whole)
            I_near = (MAE_whole-radius<=y_tick & y_tick<=MAE_whole+radius);
        end
        if ~isempty(MAE_white)
            if isempty(I_near)
                I_near = ones(length(y_tick),1);
            end
            I_near = I_near&(MAE_white-radius<=y_tick & y_tick<=MAE_white+radius);              
        end    
        if nnz(I_near)
            y_tick(I_near) = [];    
        end
        % unify the MAE_whole and MAE_white as new ticks
        y_tick = sort([y_tick MAE_whole MAE_white]);  
        set(h2,'YTick',y_tick);
    end
end    
    % 2.6 set the Y-axis right
if flag_plot_MAE_line&&flag_barPlot
  set(h2,'YAxisLocation','right','Color','none');
  set(h2,'XLim',get(h1,'XLim'),'Layer','top');  
end
    % 2.7 set xlabel and ylabel
if ~isempty(indexName)    
    xlabel(gca,indexName,'fontsize',labelFontSize);
end

if flag_plot_MAE_line && flag_barPlot    
    ylabel(h2,'MAE ','fontsize',labelFontSize);
elseif ~flag_plot_MAE_line && flag_barPlot    
    ylabel(gca,'number of entries','fontsize',labelFontSize);    
elseif flag_plot_MAE_line && ~flag_barPlot    
    ylabel(gca,'MAE ','fontsize',labelFontSize);
end
    % 2.8 set box of h1 off
if flag_plot_MAE_line && flag_barPlot    
    box(h1,'off');  
end
    % 2.9 reset the hold state as original
if ~flag_ishold
    hold off
end
end 