function [id varargout]= findFileRecord(key,varargin)
% find the first file record id (row number of the cell matrix), if not
% found return [];
% Inputs:
%  key: the key of the file record to search%       
%  varargin{1}: K, Optional, a nonnegative integer indicating the number of
%      the fixed order key fields; default value is defined in getconstant.m;
%       if this argument set 0 ; then all the sub-flags are viewed  
%       order sensitive;    
%       the key has the following format (consists with sub-flags, split by '_'):
%                      flag1_flag2_..._flagN
%       where the first K sub-flags (flag1,...,flagK) is order sensitive, while
%         the remain sub-flags are not order sensitive; 
%         e.g. if K = 2; 
%               we will view  the key 'ab_cd_ef_gh' the same as
%               'ab_cd_gh_ef'; while it is different with 'cd_ab_ef_gh'
%       more over all the keys are not case sensitive;
% Outputs:
%  id: row number of the cell matrix corresponding to the first fined
%    record (each row of the cell array is a record) , if not 
%    found, set [];
%  varargout{1}: the cell array of the finded record

% initialization
id = [];
if nargout>1
    varargout{1}=[];
end

if nargin<=1
    K = getconstant('orderSensitiveFlag');
else 
    K = varargin{1};
end

if ~exist(getconstant('fileInf'),'file')
    return
end
   
load(getconstant('fileInf'),'F'); % load file information cell matrix F
key = lower(key); % key is not case sensitive

% separate the key string 
sepstr = '_'; % separate string of the key
key_cell = textscan(key,'%s','delimiter',sepstr); % split the key string    
key_cell = key_cell{:}; 
len_key_cell = length(key_cell);

% search the record
if K<=0 || K>=len_key_cell-1 % K>=len_key_cell-1: means all the sub-flags are order sensitive
                             % K<=0: including the defualt value K=0
    for ii=1:size(F,1)
        if strcmpi(key,F{ii,2})
            id = ii;
            break;
        end
    end
else % K>0 && K<len_key_cell-1
    % search possible keys corresponding to the given field1 and sample set    
    filePredix = strcatSep(key_cell(1:K), sepstr);
    len_pre = length(filePredix);
    tf_v = strcmp(filePredix,cellfun(@(str) str(1:min(len_pre,length(str))), F(:,2),'UniformOutput',false) ); 
        % compare the string filePredix with  the first 1:len character of the strings in A(:,2)     
    id_v = find(tf_v);
    % search the key corresponding to the given flags
    for ii=1:length(id_v) 
        str_cell = textscan(F{id_v(ii),2},'%s','delimiter',sepstr); % split string    
        str_cell = str_cell{:}; % str_cell has a form {{ str1 str2 ... strN}}        
        len_str_cell = length(str_cell);          
        if len_str_cell == len_key_cell           
            if len_str_cell==K || len_str_cell>K && issubseti(str_cell(K+1:len_str_cell),key_cell(K+1:len_key_cell))
                id = id_v(ii);
                break; 
            end
        end        
    end
end % end if

if ~isempty(id) && nargout>1
    varargout{1} = F(id,:);
end

end % end of the function

function flag = issubseti(A,B)
% return whether A is a subset of B, not case sensitive
% A, B are two vectors or two cell arrays of strings
flag = 0;
if isempty(A) 
    flag = 1; % emptyset is a subset of arbitrary set
    return
end
B = cellfun(@lower,B,'UniformOutput',false);
A = cellfun(@lower,A,'UniformOutput',false);
for ii=1:length(A)
    if ~ismember(A(ii),B)
        return
    end
end
flag = 1;
end

function catstr = strcatSep(str_cell, sepstr)
    % conver a cell array STR_CELL (a row cell array) to a string,
    % separated by the string  STRSEP
    if length(str_cell)<=1
        catstr = str_cell{:};        
    else
        len_str_cell = length(str_cell);
        len_str_cell_cat = len_str_cell*2-1;
        str_cell_cat = cell(1,len_str_cell_cat);
        str_cell_cat(1:2:len_str_cell_cat) = str_cell;
        str_cell_cat(2:2:len_str_cell_cat) = {sepstr};
        catstr = cell2mat(str_cell_cat);
    end
    
    
end