function [MAE acc1 acc2 BadEntryRate  ErrorRate RMSE] = indicator_precision(II,JJ,Predict,Test,BETA)
% calculate prediction precision
% Inputs:
%  II,JJ, Predict: 1) If II, JJ, Predict are column vectors, (II(i),JJ(i))
%      indicating the predicted value for the  i-th  entries  i.e. the rating
%      of user II(i) on Item JJ(i),  Predict(i) indicates the traing grade for the i-th 
%      training  entry;
%      2) if Predict is matrix, II, JJ is useless, they can be set [];
%  Test: testing rating matrix; or a column vector with the same length as
%   Predict;
%  BETA: paddings to fill the missing values in Test matrix or the trained
%   matrix

%                     Recommended          Not Recommended
%       Prefered     True Positive (tp)    False Negative (fn)
%       Not Prefered  False Positive (fp)  True Negative (tn)
%           Fig. four cases a recommend of a item to a given user
% Outputs:
%  MAE: mean absolute error
%  acc1: accurate in the sense of bi-classifacation, like: grade 1, 2;
%   dislike: grade 3,4,5;
%  acc2: accurate in the sense of bi-classifacation, like: grade 1, 2, 3;
%   dislike: grade 4,5;
%  BadEntryRate: # bad entries (with MAE >=2) / # testing entries   
%  ErrorRate: # error entries
%  (predicted>=4&test<=2)||predicted<=2&test>=4)/# testing entries
%  ErrorEntry: a 0-1 vector as the same lenght as Test if both Test and
%   Predict are two vectors, otherwise it is a 0-1 matrix;

%  RMSE: root of mean sqare error
% %  Precision: # true positive entries / (#tp + #fp)
% %  Recall: # true positive entries / (#tp + #fn)

isMat_Predict = (size(Predict,1)~=1 && size(Predict,2)~=1); % indicat whether Predict is a matrix
isMat_Test = (size(Test,1)~=1 && size(Test,2)~=1); % indicat whether Test is a matrix

% calculate the indicators
  % case 1.  both Test and Predict are 1-dimensional vectors
if ~isMat_Test && ~isMat_Predict 
    NUM = length(Test);
    MAE  = sum(abs(Predict-Test))/NUM;
    RMSE = sqrt(sum((Predict-Test).^2)/NUM);
    ErrorRate = nnz(Predict>=4 & Test<=2 | Predict<=2 & Test>=4)/ NUM;
    acc1 = nnz( Predict<=2 &(Test<=2) | (Predict>2) &(Test>2) )/NUM;
    acc2 = nnz(  Predict<=3 &(Test<=3) |(Predict>3) &(Test>3) )/NUM;
    BadEntryRate = nnz( abs(Predict-Test)>=2 )/NUM;
    return;
end

% other cases: either Test or Predict is a matrix
if ~isMat_Predict
    Predict = sparse(II,JJ,Predict);
    Predict = (Predict==0)*BETA + Predict; % fill the untrained entries into BETA
end
if ~isMat_Test
    Test = sparse(II,JJ,Test);
    Test = (Test==0)*BETA + Test; % fill the untrained entries into BETA
end

% calulate MAE, acc1, acc2;
[nUser0 nItem0] = size(Predict);
[nUser1 nItem1] = size(Test);
nUser  = min(nUser0,nUser1);
nItem = min(nItem0, nItem1);
Predict = Predict(1:nUser,1:nItem);
Test = Test(1:nUser,1:nItem);

I = (Predict~=BETA)&(Test~=BETA);
NUM = max(1,nnz(I));
MAE_matrix = abs((Predict-Test).*I);
MAE = sum(sum(MAE_matrix))/nnz(I);
MAE = full(MAE);
RMSE = sqrt(sum(sum(MAE_matrix.^2))/nnz(I));
RMSE = full(RMSE);

ErrorRate = nnz((Predict>=4 & Test<=2 | Predict<=2 & Test>=4).*I)/ NUM;
BadEntryRate = nnz( abs((Predict-Test).*I)>=2 )/NUM;
acc1 = nnz( (Predict<=2 &(Test<=2) | (Predict>2) &(Test>2) ) .*I)/NUM;
acc2 = nnz( ( Predict<=3 &(Test<=3) |(Predict>3) &(Test>3) ) .*I)/NUM;
end
