function [MAE acc1 acc2 BadEntryRate  ErrorRate MAE_dist relatedness_dist] = indicators_index(II,JJ,relatedness,Test,Predict,BETA,varargin)
% calculate the prediction precision for a trained rating matrix
% 
% Inputs:
%  II,JJ: column vector, (II(i),JJ(i)) indicating the i-th trained entries;
%       i.e. the rating of user II(i) on Item JJ(i)
%  relatedness: column vector, indicating the bi_relatedness for the i-th training entry;
%  Test: 1) testing rating matrix; or 2) a column vector with the same
%   length as II and JJ
%  Predict: predicted values for the entries on the test set,
%   1) if Predict is a column vector with the same length as II and JJ, Predict(i) indicate the 
%   prediction for the rating of user II(i) on item JJ(i); 
%   2) if Predict is a matrix, Predict(i,j) indicate the prediction for the rating of user
%    i on item j; 
%  BETA: paddings to fill the missing values in Test matrix or the trained
%   matrix
%
%                     Recommended          Not Recommended
%       Prefered     True Positive (tp)    False Negative (fn)
%       Not Prefered  False Positive (fp)  True Negative (tn)
%           Fig. four cases a recommend of a item to a given user
%  varargin{1}:  bag_num_rel,  Optional, the number of sections of relatedness when to
%   calculate relatedness_dist;
%  varargin{2}: coverage: Optional, a 2*1 vector or a positive scalar, 
%    (1) a 2*1 vector, indicating the statistical entries with
%       their relatedness located between [coverage(1) coverage(2)]
%    (2) a scalar on the interval (0,1): the percentage of calculated
%    testing entries in the distribution MAE_dist and relatedness_dist
% Outputs:
%  MAE: mean absolute error
%  acc1: accurate in the sense of bi-classifacation, like: grade 1, 2;
%   dislike: grade 3,4,5;
%  acc2: accurate in the sense of bi-classifacation, like: grade 1, 2, 3;
%   dislike: grade 4,5;
%  BadEntryRate: # bad entries (with MAE >=2) / # testing entries   
%  ErrorRate: # error entries
%  (predicted>=4&test<=2)||predicted<=2&test>=4)/# testing entries
%  Precision: # true positive entries / (#tp + #fp)
%  Recall: # true positive entries / (#tp + #fn)
%  MAE_dist: a p * 2 matrix, indicate the number of testing elements
%    between various MAE sections;
%    Eg. MAE_dist= [1.0, 3;
%                     2.0, 5;
%                     3.0, 3;
%                     4.0, 7]; 
%    means the number of testing points with MAE lies in
%    (-inf,1.0],(1.0,2.0], ..., (3.0,4.0] are 3, 5, 3, 7, respectively.
%  bi_relatedness:  a q * 4 matrix, indicating the number of testing
%   entries lies between various bi_relatedness sections
%  Eg.   bi_relatedness = [ 0.020, 3, 0.52 0.090;
%                           0.025, 25, 0.60 0.062;
%                           0.030, 122, 0.65 0.065;
%                           0.035, 211, 0.67 0.063];
%    means the number of testing entries lie in (-inf,0.020],
%    ...(-0.030,0.035] are 3, ..., 211, the corresponding MAE of the
%    entries lying in these four sections are 0.52, ..., 0.67, and the
%    ErrorRate of these four sections are  0.090, ..., 0.063;

% parameters
bags = (0.5:0.5:4)';
if nargin<=6
    bag_num_rel = 21; 
else
    bag_num_rel =varargin{1};
end
if nargin<=7
    coverage = 1;
else
    coverage = varargin{2};
end

% check input parameters
isMat_Predict = (size(Predict,1)~=1 && size(Predict,2)~=1); % indicat whether Predict is a matrix
isMat_Test = (size(Test,1)~=1 && size(Test,2)~=1); % indicat whether Test is a matrix

% check input parameters
NUM = length(II);
if NUM~=length(JJ)
	error('the length of II is %d, it should be equal to the length of JJ (now it is %d).',NUM,length(JJ));
end
if ~isempty(Predict)&&~isMat_Predict
    if NUM~=length(Predict)
         error('the length of II is %d, it should be equal the length of Predict (now it is %d).',NUM,length(Predict));
    end
end
if ~isMat_Test
	if NUM~=length(Test)
        error('the length of II is %d, it should be equal the length of Test (now it is %d).',NUM,length(Test));
    end    
end
if NUM~=length(relatedness)
    error('the length of II is %d, it should be equal the length of RELATEDNESS (now it is %d).',NUM,length(relatedness));
end


if ~isempty(Predict)&&~isMat_Predict
    Predict = full(sparse(II,JJ,Predict));
    Predict = (Predict==0)*BETA + Predict; % fill the untrained entries into BETA
elseif isempty(Predict) 
    error('Predict is required.');
end
if ~isMat_Test
    Test = sparse(II,JJ,Test);
    Test = (Test==0)*BETA + Test; % fill the untrained entries into BETA
end


% calulate MAE, acc1, acc2;
[nUser0 nItem0] = size(Predict);
[nUser1 nItem1] = size(Test);
nUser  = min(nUser0,nUser1);
nItem = min(nItem0, nItem1);
Predict = Predict(1:nUser,1:nItem);
Test = Test(1:nUser,1:nItem);

I = (Predict~=BETA)&(Test~=BETA);
MAE_matrix = abs((Predict-Test).*I);
MAE = full(sum(sum(MAE_matrix))/nnz(I));
ErrorRate = nnz( (Predict>=4 & Test<=2 | Predict<=2 & Test>=4).*I)/ max(1,nnz(I));
acc1 = nnz( (Predict<=2 &(Test<=2) | (Predict>2) &(Test>2) ) .*I)/nnz(I);
acc2 = nnz( ( Predict<=3 &(Test<=3) |(Predict>3) &(Test>3) ) .*I)/nnz(I);

% calculate the distribution of MAE, and BadEntryRate
ind = find(I); % note that ind is a column vector
A = abs(Predict-Test);
x = A(ind);
[num_balls]= falltoBags(x,bags);
MAE_dist = [bags num_balls];
BadEntryRate = sum(num_balls(5:8))/sum(num_balls);

% calculate the distribution of bi_relatedness corresponding to MAE
rel = relatedness;
Rel = full(sparse(II,JJ,rel));
Rel = (~Rel)*BETA + Rel; % fill the untrained entries into BETA
rel = Rel(ind);

if isscalar(coverage)      
	[num_rel indices_bags bags] = distribute(rel,[],bag_num_rel,coverage,'center');
else % coverate is a 2-by-1 array
    bags = linspace(coverage(1),coverage(2),bag_num_rel)';
    [num_rel indices_bags]= falltoBags(rel,bags);
end

MAE_vector = myaccumarray(indices_bags,full(MAE_matrix(ind)),bag_num_rel)./max(num_rel,1);
ErrorRate_vector = myaccumarray(indices_bags,full(Predict(ind)>=4&Test(ind)<=2|Predict(ind)<=2&Test(ind)>=4),bag_num_rel)...
    ./ max(1,num_rel);
relatedness_dist = [bags num_rel MAE_vector ErrorRate_vector];

end


function f = myaccumarray(ind,v,nbags)
% retain NBAGS bags, fill the blank bags to zero or discard remaining bags
maxBagInd = max(ind);
if maxBagInd<=nbags
    f = zeros(nbags,1);
    f(1:maxBagInd)= accumarray(ind,v);
else    
    f = accumarray(ind,v);
    f = f(1:nbags);
end
end