function [U I x y iteInfUser iteInfItem iteInform] = activeSubmatrix(A,b,c,arg)
% find a submatrix (local community) A(U,I) from A 
% Inputs:
%  A: a m*n (full) matrix 
%  b: a column vector with length m;
%  c: a column vector with length n;
%  arg: Optional. 
%   arg.machine: Optional, a string indicated the method used to explore
%     related users and items; 
%     'Homotopy': use generalized Homotopy method (by default);
%     'Pearson': use Pearson Similarity;
%   arg.parCheck:  Optional, whether check the size input parameters A, b and c, (1 or
%                  0), default 0;
%   arg.aveR    :  Optional, a 4*1 cell arrary consisting average ratings of users of
%       rating matrix A, items for rating matrix A, item b, user c,
%       respectively; this is only required for the machine 'Pearson';
%   arg.style:     Optional, 'adaptive' or 'static', indicating adaptive or
%       static local community size; default 'adaptive'; 
%   arg.submatrixSize: Optional, a 1 * 2 vector indicating the size of the
%       local community to search, this is only required when arg.style is set 
%       'static';
% % %   arg.aveLen: Optional, expected average length of local communities on all the tested
% % %       ratings, effective when arg.style is set 'adaptive' (default
% % %       value); the default value of arg.aveLen = 25;
%   arg.n1, n2, r0, r1, r2: parameters for function makefun_gamma()
%   arg.kappa_alpha_coef: coefficient (input parameter) for function makefun_kappa() 
%   arg.kappa_beta_coef:  coefficient (input parameter) for function makefun_kappa()
%  Outputs:
%  U, I: set of indices indicating the active submatrix A(U,I);
%  x, y: store regress coefficients along rows and columns corespongding U
%       and I, repectively for machine 'Homotopy'; for machine 'Pearson',
%       x, y are similarities of  each selected users and rating vector c,
%        each selected items and rating vector b;
%       note that x is a column vector with the same length as U;
%       y is a column vector with the same lenght as I; 
%  iteInfUser: for machine 'Homotopy', this is a structure as follows, for machine
%       'Pearson', all the fields are set 0 except '.grade' field;
%       iteInfUser.lambda: the value of lambda at iteration along
%                rows regression;
%       iteInfUser.residual: 2-norm of the residual at iteration
%                along rows regression;
%       iteInfUser.solutionNorm: 1-norm of the solution (with unitized
%              data!)
%       iteInfUser.fval:  objective function value along rows
%                regression (with unitized data!) 
%       iteInfUser.grade: grade predicted along rows regression 
%       
%  iteInfItem: for machine 'Homotopy', this is a structure, for machine
%       'Pearson', all the fields are set 0 except '.grade' field;
%       iteInfItem.lambda: the value of lambda at iteration along
%                columns regression
%       iteInfItem.residual: 2-norm of the residual at iteration
%                along columns regression;
%       iteInfUser.solutionNorm: 1-norm of the solution (with unitized
%              data!)
%       iteInfUser.fval:  objective function value along rows
%                regression (with unitized data!) 
%       iteInfItem.grade: grade predicted along columns regression;
%  iteInform: iteration information
% Description:
%  find a submatrix A1 of A,
%    min_{x,y}  0.5 *|A1*x-b1|^2 + lambdaStop *|x|_1 
%             + 0.5 *|A1'*y-c1|^2 + lambdaStop *|y|_1
%      s.t.     |x_i|<=kappa,  |y_j|<=kappa; for all i, j
%  such that the size (the number of both rows and columns) lie between
%     Llength and Ulength 

[m n] = size(A);
% initialization
grade_user = 0; 
grade_item = 0;
flag_user = 0; % whether the regression from user has finished
flag_item = 0;  % whether the regression from item has finished
iteInform = '';
iteInfUser = struct('lambda',0,'residual',0,'solutionNorm',0,'fval',0,'grade',0);
iteInfItem = iteInfUser;
x = zeros(m,1);
y = zeros(n,1);
U = 1:m;
I = 1:n;
ite = 0;
userlen = m;
itemlen = n;

if nargin<=3||~isfield(arg,'parCheck')
    parCheck = 1;
else
    parCheck = arg.parCheck;
end
if nargin<=3||~isfield(arg,'machine')
    machine = 'Homotopy';
else
    machine = arg.machine;
end
if nargin<=3||~isfield(arg,'style')||~isfield(arg,'submatrixSize')||~strcmpi(arg.style,'static')
    isStaticSize = 0;
% %     if nargin<=3||~isfield(arg,'aveLen')
% %         aveLen = 20;
% %     else
% %         aveLen = arg.aveLen;
% %     end
else
    isStaticSize = 1;
end
if parCheck
    if m~=length(b) || size(b,2)~=1
        error('the row number of matrix A should equal the length of b, and b should be a column vector.\n');
    end
    if n~=length(c) || size(c,2)~=1
        error('the column number of matrix A should equal the length of c, and c should be a column vector.\n');
    end    
    if isempty(A)
        fprintf('Empty rating matrix when calculate active Submatrix.\n');
        return;
    end
end

switch lower(machine)
    case 'homotopy'
        id_machine = int8(1);
    otherwise % 'Pearson' etc.
        id_machine = int8(2);
        aveRu = arg.aveR{1};
        aveRi = arg.aveR{2};
        aveRb = arg.aveR{3};
        aveRc = arg.aveR{4};
end

% parameters
re = 0.8; % reducing parameter for user/item numbers
detail = 1; % whether assign the iteration information to iteInform
MAX_ITE = max([0.5*m,0.5*n,3]);
 % [r1, r2]: the size of the local community to search 
if ~isStaticSize
    adaptive_gamma = makefun_gamma(arg);
    r1 = round(m*adaptive_gamma(m));
    r2 = round(n*adaptive_gamma(n));
else
    r1 = arg.submatrixSize(1);
    r2 = arg.submatrixSize(2);
end

% function handle to calculate adaptive_kappa_alpha and adaptive_kappa_beta
fun_kappa_alpha = makefun_kappa(arg.kappa_alpha_coef); 
fun_kappa_beta = makefun_kappa(arg.kappa_beta_coef);

% 1. choose local community
while 1    
    % stop creteria
    if flag_item && flag_user
        break;
    end
    if ite>MAX_ITE
        if detail
          iteInform = sprintf('%s Exceed maximum iteration number.\n',iteInform);            
        end
        break;
    end
    % 1. choose columns
    if ~flag_item 
        ite = ite + 1;
        if ~flag_user && itemlen*re>r2
            itemlen = round(re*itemlen);                
        else
            itemlen = r2;                
        end
        if id_machine==int8(1)
            [y,actSet,iteInfItem] = solveLasso_kCovariant(A(U,I),b(U), itemlen, fun_kappa_beta);                        
            grade_item = full(c(I)'*y);      % prediction for the grade, note that length(y) = length(I)       
            I = I(actSet);
        else 
            [y,actSet] = pearson_kCovariant(A(U,I),b(U),itemlen,[aveRi(I);aveRb]);
            I = I(actSet);
            grade_item = aveRb + sum((c(I)-aveRi(I)).*y)/norm(y,1); % length(y) == length(actSet)
        end        
        if itemlen<=r2;
            flag_item = 1;
        end        
        printfInf();
    end
    % 2. choose rows 
    if ~flag_user
        ite = ite + 1;
        if ~flag_item && userlen*re>r1
            userlen = round(re*userlen);
        else
            userlen = r1;
        end
        if id_machine==int8(1)            
            [x,actSet,iteInfUser] = solveLasso_kCovariant(A(U,I)',c(I), userlen, fun_kappa_alpha);            
            grade_user = full(x'*b(U)); % note that length(x) == length(U)
            U = U(actSet);
        else
            [x,actSet] = pearson_kCovariant(A(U,I)',c(I),userlen,[aveRu(U); aveRc]);
            U = U(actSet);
            grade_user = aveRc + sum((b(U)-aveRu(U)).*x)/norm(x,1); % length(x) == length(actSet)
        end        
        if userlen<=r1
            flag_user = 1;
        end        
        printfInf();
    end        
end % end while

% ending
if ~isempty(grade_user)
    iteInfUser.grade = grade_user;    
else
    iteInfUser.grade = 0;    
end
if ~isempty(grade_item)   
    iteInfItem.grade = grade_item;
else
    iteInfItem.grade = 0;
end

    function printfInf()
      if detail        
        frm='%siter:\t%d\tgrade_user:\t%.1f\tgrade_item:\t%.1f\t\tlength(U):%d\tlength(I):%d\n';    
        iteInform = sprintf(frm,iteInform,ite,grade_user,grade_item,length(U),length(I));
        %%%
        %  print the information to command window
        %fprintf(frm,'',ite,grade_user,grade_item,length(U),length(I));            
        %%%
      end
    end
end % end of activeSubmatrix()

%%%%%%
% % function gamma = adaptive_gamma(n,aveLen)
% % % generate adaptive gamma  
% % % Inputs 
% % %   n: number  of users (or the items) of the matrix where to search the local community;
% % %   aveLen: expected average length of local communities on all the tested
% % %       ratings;
% % % Outputs 
% % %   gamma:  a scalar located in (0,1), indicating the number of the users
% % %      or items to search: n*gamma.
% % 
% % rr0 = 0.8;
% % rr1 = 0.6;   % rr1, rr2 in (0,1), rr2<rr1
% % n1 = aveLen; % if n==n1, then r1=rr1*n1;
% % rr2 = 0.4;
% % n2 = 3*n1; % if n==n2, then r1=rr2*n2; 
% %            % MAKE SURE THAT rr2,n2 satisfies that n2*rr2>rr1*n1.
% % 
% % p = log(rr1/rr2)/log((n1+1)/(n2+1));
% % alpha = rr1*(n1+1)^(-p);
% % if n<=n1
% %     gamma =  (rr1-rr0)/n1*n + rr0; 
% %             % a line pass through the two points (0, rr0), (n1, rr1)            
% % else
% %     gamma = alpha*(n+1).^p;
% %             % alpha,p satisfies that the curve pass through the two points:
% %             %  (n1, rr1), (n2, rr2);
% % end
% % end
%%%%%