function [grade inform]= predictOneElement(dataFile,ii,jj,arg)
% predict the rating by a user for a certain item
% Inputs:
%  datafile: data file name
%  ii,jj: the indices of user and item, respectively
%  arg: Optional, a structure for setting arguments; see activeSubmatrix
%   for detail; here, arg may has an appended field (Optional),
%       arg.dataFileInf: name of the file containing variable:
%           ave_rating_item, ave_rating_user, which is the average rating
%           of users and items, only used for arg.machine is set 'Pearson'
%       arg.predictFile: name of the mat file containing the predictions
%           of the missing ratings;
%       arg.n1, n2, r0, r1, r2: parameters for function makefun_gamma()
%       arg.kappa_alpha: parameter for function makefun_kappa() 
%       arg.kappa_beta:  parameter for function makefun_kappa()
% Outputs:
%  grade: predicted rating
%  inform: a structure at least contains the fields as follows
%     inform.grade = grade (the first output argument); 
%     inform.trainSize: size of the training matrix (the one to search the
%        local community) 
%     inform.nnz: number of nonzero elements (known ratings) of the training matrix;
%     inform.density: inform.nnz/[inform.trainSize(1)*inform.trainSize(2)];
% 	  inform.itemSolutionNorm: 1-norm of the solution of the l1-regression
%       model for items  
%     inform.userSolutionNorm: 1-norm of the solution of the l1-regression
%       model for users  
%     inform.userResidual: residual of the l1-regression model for users  
%     inform.itemResidual: residual of the l1-regression model for items
%     inform.relatedness: relatedness for the searched user-item submatrix
%       (local community);
%     inform.userLambda: lambda of the l1-regression model for users at termination; 
%     inform.itemLambda: lambda of the l1-regression model for items at termination;  
%     inform.U: indices of rows of the user-item submatrix (local community)
%     inform.I: indices of columns of the local community
%     inform.sizeSubmatrix: size of the local community
%     inform.iteInf: iteration information
%     inform.densitySubmatrix: density of known ratings  of the user-item submatrix 
%     ------------------------------------------------------------------

persistent A A1 fileTrain MAX_GRADE MIN_GRADE MID_GRADE avei aveu
    % NOTE THAT these variables are PERSISTENT.

% parameters
BETA = 0;
flag_fillMissNearNbr = 1; 
    % whether fill the missing ratings of the rating matrix by nearest
    % neighbor method 

% initialization
if isempty(A) || ~strcmp(fileTrain,dataFile)
    fileTrain = dataFile;    
    load(fileTrain,'Train');
    A = Train;
    I = A~=BETA;
    MAX_GRADE = full(max(A(I))); % maximum rating
    MIN_GRADE = full(min(A(I))); % minimum rating
    MID_GRADE = MIN_GRADE + (MAX_GRADE-MIN_GRADE)/2; % median rating
    if flag_fillMissNearNbr
        %predictFile = '../iteration/NearestNeighbor/predict_nNbr.mat';
        %predictFile = getFileName('ua','Test','nNbr','user');        
        load(arg.predictFile,'Predict');
        A1 = A.*I + ~I.*Predict;
    else
        A1 = A.*I + ~I*MID_GRADE; % fill the missing ratings as  median rating
    end
end
if isempty(avei) && strcmpi(arg.machine,'pearson') && isfield(arg,'dataFileInf')
    load(arg.dataFileInf,'ave_rating_item','ave_rating_user');
    avei = ave_rating_item;
    aveu = ave_rating_user;
end


inform=struct('grade',MID_GRADE,'userGrade',MID_GRADE,'itemGrade',MID_GRADE,...
    'trainSize',[0,0],'nnz',0,'density',0.0,'itemSolutionNorm',BETA,...
    'userSolutionNorm',BETA,'userResidual',BETA,'itemResidual',BETA,...
    'relatedness',1,'userRel',[],'itemRel',[],'userLambda',BETA,...
    'itemLambda',BETA,'sizeSubmatrix',[0,0],'iteInf','','densitySubmatrix',BETA);
grade = MID_GRADE; 

if ii>size(A,1) || jj>size(A,2)
    fprintf('exceeded the size of training set.\n');
    return;
end
I = find(A(ii,:)~=BETA);
I = setdiff(I,jj);
U = find(A(:,jj)~=BETA);
U = setdiff(U,ii); % declude the ii-th user from U
I = I(sum(A(U,I)~=BETA,1)~=0); % declude users and items with no ratings
U = U(sum(A(U,I)~=BETA,2)~=0);
if isempty(U)|| isempty(I)
    fprintf('no rated items or no other users rated in this item.\n');
    return;
end

inform.trainSize = [length(U),length(I)];
inform.nnz = nnz(A(U,I)~=BETA);
inform.density = inform.nnz/(length(U)*length(I));

% 1) search the local community
if strcmpi(arg.machine,'pearson')
    arg.aveR = {aveu(U),avei(I),avei(jj),aveu(ii)};
end
[U1 I1 x y infUser infItem iteInf] = activeSubmatrix(full(A1(U,I)),full(A1(U,jj)),full(A1(ii,I)'),arg);
U = U(U1);
I = I(I1);

% 2) calculate relatedness
evaluate_inform();
grade = inform.grade;

    function evaluate_inform()        
        inform.iteInf = iteInf;
        inform.userLambda = infUser.lambda;
        inform.itemLambda = infItem.lambda;
        inform.userResidual = infUser.residual;
        inform.itemResidual = infItem.residual;        
        inform.userSolutionNorm = infUser.solutionNorm;
        inform.itemSolutionNorm = infItem.solutionNorm;
        inform.userRel = infUser.fval;
        inform.itemRel = infItem.fval;
        inform.userGrade = infUser.grade;
        inform.itemGrade = infItem.grade;
        inform.sizeSubmatrix = [length(U1) length(I1)];
        inform.densitySubmatrix = nnz(A(U,I)~=BETA)/(length(U1)*length(I1)); 
        if strcmpi(arg.machine,'homotopy')
            inform.relatedness = unrel2rel(max(inform.userRel,inform.itemRel));
        else % strcmpi(arg.machine,'pearson')
            inform.relatedness = min(mean(abs(x)),mean(abs(y)));
        end
            % choose the prediction of the rating value
        ts1 = inform.trainSize(1);
        ts2 = inform.trainSize(2);
        %if related_err(infUser.grade,infItem.grade)>=0.5 && ts1>ts2 || outlier(inform.itemGrade,MIN_GRADE,MAX_GRADE)
        if related_err(ts1,ts2)>=0.6 && min(ts1,ts2)<=25 || outlier(inform.itemGrade,MIN_GRADE,MAX_GRADE)
            inform.grade = clip(inform.userGrade); 
        else
            inform.grade = clip(inform.itemGrade);
        end
    end
    function t1=clip(t)
        t1=min(max(t,MIN_GRADE),MAX_GRADE);
    end   
end % end of the function predictOneElement()


function err = related_err(t1,t2)
    err = abs(t1-t2)/max(abs(t1),abs(t2));
end
    
function flag = outlier(t,lb,ub)
% check whether t is far away from the range [lb, ub]        
    if t<=ub&&t>=lb
        flag = 0;
    elseif t<lb
        flag = abs(t-lb)/max(abs(t),abs(lb))>0.8;
    else % t>ub
        flag = abs(ub-t)/max(abs(t),abs(ub))>0.8;
    end        
end

function coh_v = unrel2rel(unrel)
% transform the unrelatedness to relatedness (or coherence)
    unrel = unrel + ~unrel; % set the unrel=1 if unrel = 0;
    coh_v = -log(min(unrel,1));
end