function [x,activeSet_whole,inform] = solveLasso_kCovariant(A, y, k, fun_kappa)
%   choose k columns of A most related with vector y, unitize each column of A and y before
%     operation
%Inputs
%	A    : a  matrix with size [m n].
%	y    : column vector with length m.
%   k    : choose k columns of A, most related with y, if possible;
%  fun_kappa: a function handle to calculate the parameter kappa, with the
%         format:  kappa = adaptive_kappa(n)
%       with n the dimension of the solution space;
% Outputs
%	x               solution of the original (NOT UNITIZED) problem;
%   activeSet_whole a vecter consists indices of selected columns
%   inform          iterate information, 
%                     inform.residual: L2 norm of the residual at termination (with unitized data!); 
%                     inform.lambda: lambda at termination (with unitized data!);
%                     inform.fval:   objective function value at termination (with unitized data!) 
%                     inform.solutionNorm: 1-norm of the solution (with unitized data!)
parCheck = 0;
%unitize data
[AI magA] = unitize(A);
[yI magy] = unitize(y);
% solving
[x,activeSet_whole, inform] = solveLasso_kCovariant_0(AI, yI, k, parCheck,fun_kappa);
inform.solutionNorm = norm(x,1);
inform.fval = 0.5*inform.residual^2+inform.lambda*inform.solutionNorm;
% rescaling the solution for un-unitized program
magA = magA + (abs(magA)<eps)*1.0; 
x = magy*x./magA;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [A b]= unitize(A)
% unitize A such that every column of A is a unit vector 
% b: a column vector such that b(i) = norm(A(:,i))

b=sqrt(sum(A.*A))';
for ii = 1:length(b)
    if b(ii)
        A(:,ii)=A(:,ii)/b(ii);
    end
end
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [x,activeSet_whole, inform] = solveLasso_kCovariant_0(A, y, k,parCheck,fun_kappa)
% find k columns of a matrix A which is most related with vector y
% Input
%	A    : a  matrix with size [m n].
%	y    : column vector with length m.
%   k    : choose k columns of A, most related with y, if possible;
%   parCheck: Optional, whether chech the validity of input arguments (1 or 0) 
%   fun_kappa: described as above;
% Outputs
%	x               solution of the general Lasso problem
%   activeSet_whole a vector consists indices of selected columns
%   inform          iterate information, 
%                   inform.residual: L2 norm of the residual at termination; 
%                   inform.lambda: lambda at termination;
% Description
%   The algorithm solves the general Lasso problem     
%      min   0.5|y-Ax|_2^2 + lambda*|x|_1
%      s.t.  x_i <= kappa, for all i
%   for lambda >= 0, kappa>0 as given parameters using Homotopy method with
%   parameter lambda.

if nargin<=3
    parCheck =1;
end
if parCheck
    if isempty(A)||isempty(y)
        x=[];    activeSet_whole = [];    
        inform=[];
        return;
    end
    if length(y)~=size(A,1)
        error('number of the rows of A and should equal to length of y.');
    end
end

% arguments
n = size(A,2);
max_ite = max(size(A));
lambdaStop = 0.001; % lambdaStop can be chosen near 0;
arg = struct('verbose',0,'maxActInd',k); % It is important to set arg.maxActInd = k;
%%arg = struct('verbose',1,'maxActInd',k); % It is important to set arg.maxActInd = k;

% initilization
x = [];
activeSet_whole = [];
ite_num = 0;
while 1
    ite_num = ite_num + 1;
    if ite_num>max_ite
        break;
    end
    % stop criteria
    inactiveSet_whole = setdiff(1:n,activeSet_whole);    
    if isempty(inactiveSet_whole)
        break;
    end
    
    % choose related columns by Homotopy algorithm
    len_inact = length(inactiveSet_whole);
    w = ones(len_inact,1); % constant w
    %%%kappa = adaptive_kappa(len_inact);
    kappa = fun_kappa(len_inact);
    [x1, fx1, inform1] = SolveGeneralLasso(A(:,inactiveSet_whole),y,w,lambdaStop,kappa,arg);    
%     arg.w = ones(len_inact,1); % constant w
%     arg.kappa = adaptive_kappa(len_inact);
%     [x1, fx1, inform1] = SolveGeneralLasso(A(:,inactiveSet_whole),y,lambdaStop,arg);    
    activeSet1 = inform1.activeIndices;    
    if isempty(x)
        x = x1; 
        inform = struct('residual',inform1.residual,'lambda',inform1.lambda); 
    end    
    if isempty(activeSet1)
        break;
    end    
    % collect k columns (if possible)
    len_act_whole = length(activeSet_whole);
    len_act1 = length(activeSet1);    
    if len_act_whole + len_act1 < k
        if n <= k
            activeSet_whole = 1:n;
            break;
        else
            activeSet_whole = union(activeSet_whole,activeSet1);
        end
    else % len_act_whole + len_act1 >= k
        activeSet_whole = union(activeSet_whole,activeSet1(1:k-len_act_whole));
        break;
    end
end

end % end of solveLasso_kCovariant_0()




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%