function rating_matrix_inf(R,fileName,inf)
% show the information for a rating matrix
% Inputs: 
%   R: rating matrix (a sparse matrix)
%   fileName: save the information to the file
%   inf: Optional, string indication the information (e.g. the name) of the
%     rating matrix
% save to file:
%   1. the whole stachastic indices
%      num_rating:     number of ratings; 
%      num_user:       number of users;
%      num_item:       number of items;
%      desity :         density of the known ratings = num_ratings / (num_users * num_items);
%      ave_num_rating_user: average number of ratings for users;
%      ave_num_rating_item: average number of ratings of items;
%      dist_rating:     distribution of ratings, a column vector with the
%                       same length as rating set, for example [500  1421  3621  2720
%                       2855]', indicating
%                               rating   1    2     3    4    5
%                               number  500  1421  3621 2720 2855
%      ave_rating:      a real number indicating the average of the whole rating;
%      dev_rating:      standard deviation of the whole ratings;

%  2. the stachastic indices for users and items
%      num_rating_user:  a column vector, indicating the number of ratings of each user; 
%      num_rating_item:  a column vector, indicating the number of ratings for each item;
%      ave_rating_user:  a column vector, indicating the average ratings of each user;
%      ave_rating_item:  a column vector, indicating the average ratings for each item; 
%      dev_rating_user:  a column vector, indicating the standard deviation of ratings of each user; 
%      dev_rating_item:  a column vector, indicating the standard deviation of ratings for each item; 

% Initiation
if nargin<=2
    inf = '';
end
num_user = 0; num_item = 0; density = 0;
ave_num_rating_user = 0; ave_num_rating_item = 0;
dist_rating = [0 0];
ave_rating = 0; dev_rating = 0;

num_rating_user = 0;  num_rating_item = 0;
ave_rating_user = 0; ave_rating_item = 0;
dev_rating_user = 0; dev_rating_item = 0;

%   1. the whole stachastic indices
[nUser nItem] = size(R);
num_rating = nnz(R);
if ~num_rating
    save(fileName,'inf','num_rating','num_user','num_item','density','ave_num_rating_user',...
        'ave_num_rating_item','dist_rating','ave_rating','dev_rating',...
        'num_rating_user','num_rating_item','ave_rating_user','ave_rating_item','dev_rating_user','dev_rating_item');
    return;
end
num_user = nUser;
num_item = nItem;
density = num_rating/(num_user*num_item);
ave_num_rating_user = num_rating/num_user;
ave_num_rating_item = num_rating/num_item;

  % calculate the distribution of various ratings
Z = R; ii = 0;
while nnz(Z)
    ii = ii + 1;
    id = find(Z,1,'first');
    rating = Z(id);
    I = Z==rating;
    dist_rating(ii,1) = rating;
    dist_rating(ii,2) = nnz(I);
    Z(I) = 0;
end
dist_rating = sort(dist_rating,1,'descend');
  % calculate the average and std. deviation of the whole ratings
ave_rating = dist_rating(:,1)'*dist_rating(:,2)/num_rating;
dev_rating = sqrt((dist_rating(:,1)'-ave_rating).^2 * dist_rating(:,2))/sqrt(max(num_rating-1,1));

%  2. the stachastic indices for users and items
I = R~=0;
num_rating_user = full(sum(I,2)); 
num_rating_item = full(sum(I,1)');
ave_rating_user = full(sum(R,2))./max(num_rating_user,1); 
ave_rating_item = full(sum(R,1))'./max(num_rating_item,1);
dev_rating_user = sqrt(full(sum((R-spdiags(ave_rating_user,0,num_user,num_user)*I ).^2,2)))./sqrt(max(num_rating_user-1,1));
dev_rating_item = sqrt(full(sum((R-I*spdiags(ave_rating_item,0,num_item,num_item)).^2,1)'))./sqrt(max(num_rating_item-1,1));
    
% 3. save the indices to file
save(fileName,'inf','num_rating','num_user','num_item','density','ave_num_rating_user',...
        'ave_num_rating_item','dist_rating','ave_rating','dev_rating',...
        'num_rating_user','num_rating_item','ave_rating_user','ave_rating_item','dev_rating_user','dev_rating_item');
